@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Application do
   @moduledoc """
   A module for working with applications and defining application callbacks.
@@ -54,12 +58,12 @@ defmodule Application do
   You can also change the application environment dynamically by using functions
   such as `put_env/3` and `delete_env/2`.
 
-  > #### Environment in libraries {: .tip}
+  > #### Application environment in libraries {: .info}
   >
-  > The config files `config/config.exs` and `config/runtime.exs`
-  > are rarely used by libraries. Libraries typically define their environment
-  > in the `application/0` function of their `mix.exs`. Configuration files
-  > are rather used by applications to configure their libraries.
+  > If you are writing a library to be used by other developers,
+  > it is generally recommended to avoid the application environment, as the
+  > application environment is effectively a global storage. For more information,
+  > read about this [anti-pattern](design-anti-patterns.md#using-application-configuration-for-libraries).
 
   > #### Reading the environment of other applications {: .warning}
   >
@@ -243,7 +247,7 @@ defmodule Application do
   invoked if it hasn't been done yet. Then, it checks if the dependencies listed
   in the `applications` key of the resource file are already started. Having at
   least one dependency not started is an error condition. Functions like
-  `ensure_all_started/1` takes care of starting an application and all of its
+  `ensure_all_started/1` take care of starting an application and all of its
   dependencies for you.
 
   If the application does not have a callback module configured, starting is
@@ -673,13 +677,6 @@ defmodule Application do
   > You must use this function to read only your own application
   > environment. Do not read the environment of other applications.
 
-  > #### Application environment in libraries {: .info}
-  >
-  > If you are writing a library to be used by other developers,
-  > it is generally recommended to avoid the application environment, as the
-  > application environment is effectively a global storage. For more information,
-  > read our [library guidelines](library-guidelines.md).
-
   ## Examples
 
   `get_env/3` is commonly used to read the configuration of your OTP applications.
@@ -796,6 +793,12 @@ defmodule Application do
   @doc """
   Puts the `value` in `key` for the given `app`.
 
+  > #### Compile environment {: .warning}
+  >
+  > Do not use this function to change environment variables read
+  > via `Application.compile_env/2`. The compile environment must
+  > be exclusively set before compilation, in your config files.
+
   ## Options
 
     * `:timeout` - the timeout for the change (defaults to `5_000` milliseconds)
@@ -909,8 +912,7 @@ defmodule Application do
       `:permanent`, or `:transient`. See `t:restart_type/1` for more information.
 
     * `:mode` - (since v1.15.0) if the applications should be started serially
-      (`:serial`, default) or concurrently (`:concurrent`). This option requires
-      Erlang/OTP 26+.
+      (`:serial`, default) or concurrently (`:concurrent`).
 
   """
   @spec ensure_all_started(app | [app], type: restart_type(), mode: :serial | :concurrent) ::
@@ -931,18 +933,7 @@ defmodule Application do
 
   def ensure_all_started(apps, opts) when is_list(apps) and is_list(opts) do
     opts = Keyword.validate!(opts, type: :temporary, mode: :serial)
-
-    if function_exported?(:application, :ensure_all_started, 3) do
-      :application.ensure_all_started(apps, opts[:type], opts[:mode])
-    else
-      # TODO: Remove this clause when we require Erlang/OTP 26+
-      Enum.reduce_while(apps, {:ok, []}, fn app, {:ok, acc} ->
-        case :application.ensure_all_started(app, opts[:type]) do
-          {:ok, apps} -> {:cont, {:ok, apps ++ acc}}
-          {:error, e} -> {:halt, {:error, e}}
-        end
-      end)
-    end
+    :application.ensure_all_started(apps, opts[:type], opts[:mode])
   end
 
   @doc """
