@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Config do
   @moduledoc ~S"""
   A simple keyword-based configuration API.
@@ -41,7 +45,8 @@ defmodule Config do
   > application environment is effectively a global storage. Also note that
   > the `config/config.exs` of a library is not evaluated when the library is
   > used as a dependency, as configuration is always meant to configure the
-  > current project. For more information, read our [library guidelines](library-guidelines.md).
+  > current project. For more information, see ["Using application configuration for
+  > libraries"](design-anti-patterns.md#using-application-configuration-for-libraries).
 
   ## Migrating from `use Mix.Config`
 
@@ -93,6 +98,12 @@ defmodule Config do
   (assembled with `mix release`).
   """
 
+  @type config_opts :: [
+          imports: [Path.t()] | :disabled,
+          env: atom(),
+          target: atom()
+        ]
+
   @opts_key {__MODULE__, :opts}
   @config_key {__MODULE__, :config}
   @imports_key {__MODULE__, :imports}
@@ -130,7 +141,6 @@ defmodule Config do
 
       config :logger,
         level: :warn,
-        backends: [:console]
 
       config :logger,
         level: :info,
@@ -138,12 +148,12 @@ defmodule Config do
 
   will have a final configuration for `:logger` of:
 
-      [level: :info, backends: [:console], truncate: 1024]
+      [level: :info, truncate: 1024]
 
   """
   @doc since: "1.9.0"
   def config(root_key, opts) when is_atom(root_key) and is_list(opts) do
-    unless Keyword.keyword?(opts) do
+    if not Keyword.keyword?(opts) do
       raise ArgumentError, "config/2 expected a keyword list, got: #{inspect(opts)}"
     end
 
@@ -194,11 +204,37 @@ defmodule Config do
     |> put_config()
   end
 
+  @doc """
+  Reads the configuration for the given root key.
+
+  This function only reads the configuration from a previous
+  `config/2` or `config/3` call. If `root_key` points to an
+  application, it does not read its actual application environment.
+  Its main use case is to make it easier to access and share
+  configuration values across files.
+
+  If the `root_key` was not configured, it returns `nil`.
+
+  ## Examples
+
+      # In config/config.exs
+      config :my_app, foo: :bar
+
+      # In config/dev.exs
+      config :another_app, foo: read_config(:my_app)[:foo] || raise "missing parent configuration"
+
+  """
+  @doc since: "1.18.0"
+  def read_config(root_key) when is_atom(root_key) do
+    get_config!()[root_key]
+  end
+
   @doc """
   Returns the environment this configuration file is executed on.
 
   In Mix projects this function returns the environment this configuration
-  file is executed on. In releases, the environment when `mix release` ran.
+  file is executed on. 
+  In releases, returns the `MIX_ENV` specified when running `mix release`.
 
   This is most often used to execute conditional code:
 
@@ -276,7 +312,7 @@ defmodule Config do
   end
 
   @doc false
-  @spec __eval__!(Path.t(), binary(), keyword) :: {keyword, [Path.t()] | :disabled}
+  @spec __eval__!(Path.t(), binary(), config_opts) :: {keyword, [Path.t()] | :disabled}
   def __eval__!(file, content, opts \\ []) when is_binary(file) and is_list(opts) do
     env = Keyword.get(opts, :env)
     target = Keyword.get(opts, :target)
