@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Task.Supervisor do
   @moduledoc """
   A task supervisor.
@@ -74,7 +78,7 @@ defmodule Task.Supervisor do
 
   @typedoc "Option values used by `start_link`"
   @type option ::
-          DynamicSupervisor.option()
+          GenServer.option()
           | DynamicSupervisor.init_option()
 
   @typedoc """
@@ -83,6 +87,18 @@ defmodule Task.Supervisor do
   @typedoc since: "1.17.0"
   @type async_stream_option :: Task.async_stream_option() | {:shutdown, Supervisor.shutdown()}
 
+  @typedoc """
+  Options for `async/3`, `async/5`, `async_nolink/3`, and `async_nolink/5` functions.
+  """
+  @type async_opts :: [
+          shutdown: :brutal_kill | timeout()
+        ]
+
+  @type start_child_opts :: [
+          restart: :temporary | :transient | :permanent,
+          shutdown: :brutal_kill | timeout()
+        ]
+
   @doc false
   def child_spec(opts) when is_list(opts) do
     id =
@@ -171,7 +187,7 @@ defmodule Task.Supervisor do
       The tasks must trap exits for the timeout to have an effect.
 
   """
-  @spec async(Supervisor.supervisor(), (-> any), Keyword.t()) :: Task.t()
+  @spec async(Supervisor.supervisor(), (-> any), async_opts) :: Task.t()
   def async(supervisor, fun, options \\ []) do
     async(supervisor, :erlang, :apply, [fun, []], options)
   end
@@ -193,7 +209,7 @@ defmodule Task.Supervisor do
       The tasks must trap exits for the timeout to have an effect.
 
   """
-  @spec async(Supervisor.supervisor(), module, atom, [term], Keyword.t()) :: Task.t()
+  @spec async(Supervisor.supervisor(), module, atom, [term], async_opts) :: Task.t()
   def async(supervisor, module, fun, args, options \\ []) do
     async(supervisor, :link, module, fun, args, options)
   end
@@ -280,7 +296,7 @@ defmodule Task.Supervisor do
       end
 
   """
-  @spec async_nolink(Supervisor.supervisor(), (-> any), Keyword.t()) :: Task.t()
+  @spec async_nolink(Supervisor.supervisor(), (-> any), async_opts) :: Task.t()
   def async_nolink(supervisor, fun, options \\ []) do
     async_nolink(supervisor, :erlang, :apply, [fun, []], options)
   end
@@ -299,7 +315,7 @@ defmodule Task.Supervisor do
   as the `:restart` option (the default), as `async_nolink/5` keeps a
   direct reference to the task which is lost if the task is restarted.
   """
-  @spec async_nolink(Supervisor.supervisor(), module, atom, [term], Keyword.t()) :: Task.t()
+  @spec async_nolink(Supervisor.supervisor(), module, atom, [term], async_opts) :: Task.t()
   def async_nolink(supervisor, module, fun, args, options \\ []) do
     async(supervisor, :nolink, module, fun, args, options)
   end
@@ -498,7 +514,7 @@ defmodule Task.Supervisor do
   Starts a task as a child of the given `supervisor`.
 
       Task.Supervisor.start_child(MyTaskSupervisor, fn ->
-        IO.puts "I am running in a task"
+        IO.puts("I am running in a task")
       end)
 
   Note that the spawned process is not linked to the caller, but
@@ -519,7 +535,7 @@ defmodule Task.Supervisor do
       The task must trap exits for the timeout to have an effect.
 
   """
-  @spec start_child(Supervisor.supervisor(), (-> any), keyword) ::
+  @spec start_child(Supervisor.supervisor(), (-> any), start_child_opts) ::
           DynamicSupervisor.on_start_child()
   def start_child(supervisor, fun, options \\ []) do
     restart = options[:restart]
@@ -534,7 +550,7 @@ defmodule Task.Supervisor do
   Similar to `start_child/3` except the task is specified
   by the given `module`, `fun` and `args`.
   """
-  @spec start_child(Supervisor.supervisor(), module, atom, [term], keyword) ::
+  @spec start_child(Supervisor.supervisor(), module, atom, [term], start_child_opts) ::
           DynamicSupervisor.on_start_child()
   def start_child(supervisor, module, fun, args, options \\ [])
       when is_atom(fun) and is_list(args) do
@@ -598,7 +614,7 @@ defmodule Task.Supervisor do
     case start_child_with_spec(supervisor, [get_owner(owner), :monitor], :temporary, shutdown) do
       {:ok, pid} ->
         if link_type == :link, do: Process.link(pid)
-        alias = :erlang.monitor(:process, pid, alias: :demonitor)
+        alias = Task.__alias__(pid)
         send(pid, {owner, alias, alias, get_callers(owner), {module, fun, args}})
         %Task{pid: pid, ref: alias, owner: owner, mfa: {module, fun, length(args)}}
 
@@ -614,7 +630,7 @@ defmodule Task.Supervisor do
   defp build_stream(supervisor, link_type, enumerable, fun, options) do
     shutdown = Keyword.get(options, :shutdown, 5000)
 
-    unless (is_integer(shutdown) and shutdown >= 0) or shutdown == :brutal_kill do
+    if not ((is_integer(shutdown) and shutdown >= 0) or shutdown == :brutal_kill) do
       raise ArgumentError, ":shutdown must be either a positive integer or :brutal_kill"
     end
 
