@@ -1,3 +1,7 @@
+%% SPDX-License-Identifier: Apache-2.0
+%% SPDX-FileCopyrightText: 2021 The Elixir Team
+%% SPDX-FileCopyrightText: 2012 Plataformatec
+
 %% Translate Elixir quoted expressions to Erlang Abstract Format.
 -module(elixir_erl_pass).
 -export([translate/3, translate_args/3, no_parens_remote/2, parens_map_field/2]).
@@ -244,7 +248,6 @@ translate({{'.', _, [Left, Right]}, Meta, []}, _Ann, S)
 
   case proplists:get_value(no_parens, Meta, false) of
     true ->
-      TError = {tuple, Ann, [{atom, Ann, badkey}, TRight, TVar]},
       {{'case', Generated, TLeft, [
         {clause, Generated,
           [{map, Ann, [{map_field_exact, Ann, TRight, TVar}]}],
@@ -257,7 +260,7 @@ translate({{'.', _, [Left, Right]}, Meta, []}, _Ann, S)
             {clause, Generated,
              [{tuple, Generated, [{atom, Generated, ok}, TInnerVar]}], [], [TInnerVar]},
             {clause, Generated,
-             [{var, Generated, '_'}], [], [?remote(Ann, erlang, error, [TError])]}
+             [{tuple, Generated, [{atom, Generated, error}, TInnerVar]}], [], [?remote(Ann, erlang, error, [TInnerVar])]}
           ]}]}
       ]}, SV};
     false ->
@@ -302,9 +305,16 @@ translate(Other, Ann, S) ->
 
 translate_case(Meta, Expr, Opts, S) ->
   Ann = ?ann(Meta),
-  Clauses = elixir_erl_clauses:get_clauses(do, Opts, match),
   {TExpr, SE} = translate(Expr, Ann, S),
-  {TClauses, SC} = elixir_erl_clauses:clauses(Clauses, SE),
+  Clauses = elixir_erl_clauses:get_clauses(do, Opts, match),
+  RClauses =
+    %% For constructs that optimize booleans, we mark them as generated
+    %% to avoid reports from the Erlang compiler but specially Dialyzer.
+    case lists:member({optimize_boolean, true}, Meta) of
+      true -> [{N, ?generated(M), H, B} || {N, M, H, B} <- Clauses];
+      false -> Clauses
+    end,
+  {TClauses, SC} = elixir_erl_clauses:clauses(RClauses, SE),
   {{'case', Ann, TExpr, TClauses}, SC}.
 
 translate_list([{'|', _, [Left, Right]}], Ann, List, Acc) ->
@@ -426,7 +436,7 @@ translate_with_else(Meta, [{'else', Else}], S) ->
 
   Clauses = elixir_erl_clauses:get_clauses('else', [{'else', Else ++ [RaiseClause]}], match),
   {TranslatedClauses, SC} = elixir_erl_clauses:clauses(Clauses, SV#elixir_erl{extra=pin_guard}),
-  with_else_closure(Meta, TranslatedClauses, SC#elixir_erl{extra=SV#elixir_erl.extra}).
+  with_else_closure(Generated, TranslatedClauses, SC#elixir_erl{extra=SV#elixir_erl.extra}).
 
 with_else_closure(Meta, TranslatedClauses, S) ->
   Ann = ?ann(Meta),
@@ -470,6 +480,7 @@ translate_struct(Ann, Name, {'%{}', _, [{'|', _, [Update, Assocs]}]}, S) ->
   Map = {map, Ann, [{map_field_exact, Ann, {atom, Ann, '__struct__'}, {atom, Ann, Name}}]},
 
   Match = {match, Ann, Var, Map},
+  %% Once this is removed, we should remove badstruct handling from elixir_erl_try
   Error = {tuple, Ann, [{atom, Ann, badstruct}, {atom, Ann, Name}, Var]},
 
   {TUpdate, TU} = translate(Update, Ann, VS),
@@ -617,7 +628,8 @@ translate_remote(Left, Right, Meta, Args, S) ->
         [TOne, TTwo] -> {{op, Ann, Right, TOne, TTwo}, SA}
       end;
     {inline_pure, Result} ->
-      translate(Result, Ann, S);
+      Generated = erl_anno:set_generated(true, Ann),
+      translate(Result, Generated, S);
     {inline_args, NewArgs} ->
       {TLeft, SL} = translate(Left, Ann, S),
       {TArgs, SA} = translate_args(NewArgs, Ann, SL),
@@ -680,6 +692,10 @@ inline_pure_function('Elixir.URI', encode_query) -> true;
 inline_pure_function('Elixir.URI', encode_www_form) -> true;
 inline_pure_function('Elixir.URI', decode) -> true;
 inline_pure_function('Elixir.URI', decode_www_for) -> true;
+inline_pure_function('Elixir.Version', parse) -> true;
+inline_pure_function('Elixir.Version', 'parse!') -> true;
+inline_pure_function('Elixir.Version', parse_requirement) -> true;
+inline_pure_function('Elixir.Version', 'parse_requirement!') -> true;
 inline_pure_function(_Left, _Right) -> false.
 
 % we do not want to try and inline calls which might depend on protocols that might be overridden later
@@ -700,9 +716,9 @@ generate_struct_name_guard([Field | Rest], Acc, S) ->
   generate_struct_name_guard(Rest, [Field | Acc], S).
 
 %% TODO: Make this a runtime error on Elixir v2.0
-no_parens_remote(nil, _Fun) -> error;
-no_parens_remote(false, _Fun) -> error;
-no_parens_remote(true, _Fun) -> error;
+no_parens_remote(nil, _Key) -> {error, {badmap, nil}};
+no_parens_remote(false, _Key) -> {error, {badmap, false}};
+no_parens_remote(true, _Key) -> {error, {badmap, true}};
 no_parens_remote(Atom, Fun) when is_atom(Atom) ->
   Message = fun() ->
     io_lib:format(
@@ -713,8 +729,10 @@ no_parens_remote(Atom, Fun) when is_atom(Atom) ->
   end,
   'Elixir.IO':warn_once(?MODULE, Message, 3),
   {ok, apply(Atom, Fun, [])};
-no_parens_remote(_Other, _Fun) ->
-  error.
+no_parens_remote(#{} = Map, Key) ->
+  {error, {badkey, Key, Map}};
+no_parens_remote(Other, _Key) ->
+  {error, {badmap, Other}}.
 
 parens_map_field(Key, Value) ->
   Message = fun() ->
