@@ -1,6 +1,11 @@
+%% SPDX-License-Identifier: Apache-2.0
+%% SPDX-FileCopyrightText: 2021 The Elixir Team
+%% SPDX-FileCopyrightText: 2012 Plataformatec
+
 -module(elixir_rewrite).
 -compile({inline, [inner_inline/4, inner_rewrite/5]}).
--export([erl_to_ex/3, inline/3, rewrite/5, match_rewrite/5, guard_rewrite/6, format_error/1]).
+-compile(nowarn_shadow_vars).
+-export([erl_to_ex/3, inline/3, rewrite/5, match/6, guard/6, format_error/1]).
 -include("elixir.hrl").
 
 %% Convenience variables
@@ -8,6 +13,7 @@
 -define(atom, 'Elixir.Atom').
 -define(bitwise, 'Elixir.Bitwise').
 -define(enum, 'Elixir.Enum').
+-define(float, 'Elixir.Float').
 -define(function, 'Elixir.Function').
 -define(integer, 'Elixir.Integer').
 -define(io, 'Elixir.IO').
@@ -35,16 +41,19 @@
 -define(
   rewrite(ExMod, ExFun, ExArgs, ErlMod, ErlFun, ErlArgs),
   inner_rewrite(ex_to_erl, _Meta, ExMod, ExFun, ExArgs) -> {ErlMod, ErlFun, ErlArgs};
-  inner_rewrite(erl_to_ex, _Meta, ErlMod, ErlFun, ErlArgs) -> {ExMod, ExFun, ExArgs}
+  inner_rewrite(erl_to_ex, _Meta, ErlMod, ErlFun, ErlArgs) -> {ExMod, ExFun, ExArgs, fun(ErlArgs) -> ExArgs end}
 ).
 
-erl_to_ex(Mod, Fun, Args) ->
+erl_to_ex(Mod, Fun, Args) when is_list(Args) ->
   case inner_inline(erl_to_ex, Mod, Fun, length(Args)) of
     false -> inner_rewrite(erl_to_ex, [], Mod, Fun, Args);
-    {ExMod, ExFun} -> {ExMod, ExFun, Args}
-  end.
+    {ExMod, ExFun} -> {ExMod, ExFun, Args, fun identity/1}
+  end;
 
-%% Inline  rules
+erl_to_ex(Mod, Fun, Arity) when is_integer(Arity) ->
+  inner_inline(erl_to_ex, Mod, Fun, Arity).
+
+%% Inline rules
 %%
 %% Inline rules are straightforward, they keep the same
 %% number and order of arguments and show up on captures.
@@ -145,6 +154,7 @@ inline(Mod, Fun, Arity) -> inner_inline(ex_to_erl, Mod, Fun, Arity).
 ?inline(?list, to_integer, 2, erlang, list_to_integer);
 ?inline(?list, to_tuple, 1, erlang, list_to_tuple);
 
+?inline(?map, from_keys, 2, maps, from_keys);
 ?inline(?map, intersect, 2, maps, intersect);
 ?inline(?map, keys, 1, maps, keys);
 ?inline(?map, merge, 2, maps, merge);
@@ -196,6 +206,8 @@ inline(Mod, Fun, Arity) -> inner_inline(ex_to_erl, Mod, Fun, Arity).
 ?inline(?process, unregister, 1, erlang, unregister);
 
 ?inline(?string, duplicate, 2, binary, copy);
+?inline(?string, to_atom, 1, erlang, binary_to_atom);
+?inline(?string, to_existing_atom, 1, erlang, binary_to_existing_atom);
 ?inline(?string, to_float, 1, erlang, binary_to_float);
 ?inline(?string, to_integer, 1, erlang, binary_to_integer);
 ?inline(?string, to_integer, 2, erlang, binary_to_integer);
@@ -206,8 +218,6 @@ inline(Mod, Fun, Arity) -> inner_inline(ex_to_erl, Mod, Fun, Arity).
 ?inline(?system, time_offset, 0, erlang, time_offset);
 ?inline(?system, unique_integer, 0, erlang, unique_integer);
 ?inline(?system, unique_integer, 1, erlang, unique_integer);
-
-?inline(?tuple, append, 2, erlang, append_element);
 ?inline(?tuple, to_list, 1, erlang, tuple_to_list);
 
 % Defined without macro to avoid conflict with Bitwise named operators
@@ -228,17 +238,21 @@ inner_inline(_, _, _, _) -> false.
 %%
 %% Rewrite rules are more complex than regular inlining code
 %% as they may change the number of arguments. However, they
-%% don't add new code (such as case statements), at best they
+%% don't add new code (such as case expressions), at best they
 %% perform dead code removal.
 rewrite(?string_chars, DotMeta, to_string, Meta, [Arg]) ->
   case is_always_string(Arg) of
     true -> Arg;
     false -> {{'.', DotMeta, [?string_chars, to_string]}, Meta, [Arg]}
   end;
+rewrite(erlang, _, '+', _, [Arg]) when is_number(Arg) -> +Arg;
+rewrite(erlang, _, '-', _, [Arg]) when is_number(Arg) -> -Arg;
 rewrite(Receiver, DotMeta, Right, Meta, Args) ->
   {EReceiver, ERight, EArgs} = inner_rewrite(ex_to_erl, DotMeta, Receiver, Right, Args),
   {{'.', DotMeta, [EReceiver, ERight]}, Meta, EArgs}.
 
+?rewrite(?float, to_charlist, [Arg], erlang, float_to_list, [Arg, [short]]);
+?rewrite(?float, to_string, [Arg], erlang, float_to_binary, [Arg, [short]]);
 ?rewrite(?kernel, is_map_key, [Map, Key], erlang, is_map_key, [Key, Map]);
 ?rewrite(?map, delete, [Map, Key], maps, remove, [Key, Map]);
 ?rewrite(?map, fetch, [Map, Key], maps, find, [Key, Map]);
@@ -252,8 +266,6 @@ rewrite(Receiver, DotMeta, Right, Meta, Args) ->
 ?rewrite(?process, monitor, [Arg, Opts], erlang, monitor, [process, Arg, Opts]);
 ?rewrite(?process, send_after, [Dest, Msg, Time], erlang, send_after, [Time, Dest, Msg]);
 ?rewrite(?process, send_after, [Dest, Msg, Time, Opts], erlang, send_after, [Time, Dest, Msg, Opts]);
-?rewrite(?string, to_atom, [Arg], erlang, binary_to_atom, [Arg, utf8]);
-?rewrite(?string, to_existing_atom, [Arg], erlang, binary_to_existing_atom, [Arg, utf8]);
 ?rewrite(?tuple, duplicate, [Data, Size], erlang, make_tuple, [Size, Data]);
 
 inner_rewrite(ex_to_erl, Meta, ?tuple, delete_at, [Tuple, Index]) ->
@@ -266,29 +278,32 @@ inner_rewrite(ex_to_erl, Meta, ?kernel, put_elem, [Tuple, Index, Value]) ->
   {erlang, setelement, [increment(Meta, Index), Tuple, Value]};
 
 inner_rewrite(erl_to_ex, _Meta, erlang, delete_element, [Index, Tuple]) when is_number(Index) ->
-  {?tuple, delete_at, [Tuple, Index - 1]};
+  {?tuple, delete_at, [Tuple, Index - 1], fun([Index, Tuple]) -> [Tuple, Index] end};
 inner_rewrite(erl_to_ex, _Meta, erlang, insert_element, [Index, Tuple, Term]) when is_number(Index) ->
-  {?tuple, insert_at, [Tuple, Index - 1, Term]};
+  {?tuple, insert_at, [Tuple, Index - 1, Term], fun([Index, Tuple, Term]) -> [Tuple, Index, Term] end};
 inner_rewrite(erl_to_ex, _Meta, erlang, element, [Index, Tuple]) when is_number(Index) ->
-  {?kernel, elem, [Tuple, Index - 1]};
-inner_rewrite(erl_to_ex, _Meta, erlang, setelement, [Index, Tuple, Value]) when is_number(Index) ->
-  {?kernel, put_elem, [Tuple, Index - 1, Value]};
+  {?kernel, elem, [Tuple, Index - 1], fun([Index, Tuple]) -> [Tuple, Index] end};
+inner_rewrite(erl_to_ex, _Meta, erlang, setelement, [Index, Tuple, Term]) when is_number(Index) ->
+  {?kernel, put_elem, [Tuple, Index - 1, Term], fun([Index, Tuple, Term]) -> [Tuple, Index, Term] end};
 
 inner_rewrite(erl_to_ex, _Meta, erlang, delete_element, [{{'.', _, [erlang, '+']}, _, [Index, 1]}, Tuple]) ->
-  {?tuple, delete_at, [Tuple, Index]};
+  {?tuple, delete_at, [Tuple, Index], fun([Index, Tuple]) -> [Tuple, Index] end};
 inner_rewrite(erl_to_ex, _Meta, erlang, insert_element, [{{'.', _, [erlang, '+']}, _, [Index, 1]}, Tuple, Term]) ->
-  {?tuple, insert_at, [Tuple, Index, Term]};
+  {?tuple, insert_at, [Tuple, Index, Term], fun([Index, Tuple, Term]) -> [Tuple, Index, Term] end};
 inner_rewrite(erl_to_ex, _Meta, erlang, element, [{{'.', _, [erlang, '+']}, _, [Index, 1]}, Tuple]) ->
-  {?kernel, elem, [Tuple, Index]};
-inner_rewrite(erl_to_ex, _Meta, erlang, setelement, [{{'.', _, [erlang, '+']}, _, [Index, 1]}, Tuple, Value]) ->
-  {?kernel, put_elem, [Tuple, Index, Value]};
+  {?kernel, elem, [Tuple, Index], fun([Index, Tuple]) -> [Tuple, Index] end};
+inner_rewrite(erl_to_ex, _Meta, erlang, setelement, [{{'.', _, [erlang, '+']}, _, [Index, 1]}, Tuple, Term]) ->
+  {?kernel, put_elem, [Tuple, Index, Term], fun([Index, Tuple, Term]) -> [Tuple, Index, Term] end};
 
 inner_rewrite(erl_to_ex, _Meta, erlang, 'orelse', [_, _] = Args) ->
-  {?kernel, 'or', Args};
+  {?kernel, 'or', Args, fun identity/1};
 inner_rewrite(erl_to_ex, _Meta, erlang, 'andalso', [_, _] = Args) ->
-  {?kernel, 'and', Args};
+  {?kernel, 'and', Args, fun identity/1};
 
-inner_rewrite(_To, _Meta, Mod, Fun, Args) -> {Mod, Fun, Args}.
+inner_rewrite(ex_to_erl, _Meta, Mod, Fun, Args) -> {Mod, Fun, Args};
+inner_rewrite(erl_to_ex, _Meta, Mod, Fun, Args) -> {Mod, Fun, Args, fun identity/1}.
+
+identity(Arg) -> Arg.
 
 increment(_Meta, Number) when is_number(Number) ->
   Number + 1;
@@ -302,11 +317,11 @@ increment(Meta, Other) ->
 %% The allowed operations are very limited.
 %% The Kernel operators are already inlined by now, we only need to
 %% care about Erlang ones.
-match_rewrite(erlang, _, '++', Meta, [Left, Right]) ->
+match(erlang, _, '++', Meta, [Left, Right], _S) ->
   try {ok, static_append(Left, Right, Meta)}
   catch impossible -> {error, {invalid_match_append, Left}}
   end;
-match_rewrite(Receiver, _, Right, _, Args) ->
+match(Receiver, _, Right, _, Args, _S) ->
   {error, {invalid_match, Receiver, Right, length(Args)}}.
 
 static_append([], Right, _Meta) -> Right;
@@ -322,14 +337,14 @@ static_append(_, _, _) -> throw(impossible).
 %% Guard rewrite is similar to regular rewrite, except
 %% it also verifies the resulting function is supported in
 %% guard context - only certain BIFs and operators are.
-guard_rewrite(Receiver, DotMeta, Right, Meta, Args, Context) ->
+guard(Receiver, DotMeta, Right, Meta, Args, S) ->
   case inner_rewrite(ex_to_erl, DotMeta, Receiver, Right, Args) of
     {erlang, RRight, RArgs} ->
       case allowed_guard(RRight, length(RArgs)) of
         true -> {ok, {{'.', DotMeta, [erlang, RRight]}, Meta, RArgs}};
-        false -> {error, {invalid_guard, Receiver, Right, length(Args), Context}}
+        false -> {error, {invalid_guard, Receiver, Right, length(Args), elixir_utils:guard_info(S)}}
       end;
-    _ -> {error, {invalid_guard, Receiver, Right, length(Args), Context}}
+    _ -> {error, {invalid_guard, Receiver, Right, length(Args), elixir_utils:guard_info(S)}}
   end.
 
 %% erlang:is_record/2-3 are compiler guards in Erlang which we
@@ -340,15 +355,23 @@ allowed_guard(Right, Arity) ->
   erl_internal:guard_bif(Right, Arity) orelse elixir_utils:guard_op(Right, Arity).
 
 format_error({invalid_guard, Receiver, Right, Arity, Context}) ->
-  io_lib:format("cannot invoke remote function ~ts.~ts/~B inside a ~ts",
+  io_lib:format(cannot_invoke_or_maybe_require(Receiver, Right, Arity) ++ " ~ts.~ts/~B inside a ~ts",
                 ['Elixir.Macro':to_string(Receiver), Right, Arity, Context]);
 format_error({invalid_match, Receiver, Right, Arity}) ->
-  io_lib:format("cannot invoke remote function ~ts.~ts/~B inside a match",
+  io_lib:format(cannot_invoke_or_maybe_require(Receiver, Right, Arity) ++ " ~ts.~ts/~B inside a match",
                 ['Elixir.Macro':to_string(Receiver), Right, Arity]);
 format_error({invalid_match_append, Arg}) ->
   io_lib:format("invalid argument for ++ operator inside a match, expected a literal proper list, got: ~ts",
                 ['Elixir.Macro':to_string(Arg)]).
 
+cannot_invoke_or_maybe_require(Receiver, Fun, Arity) ->
+  try
+    true = lists:member({Fun, Arity}, Receiver:'__info__'(macros)),
+    ["you must require the module", 'Elixir.Macro':to_string(Receiver), " before invoking macro"]
+  catch
+    _:_ -> "cannot invoke remote function"
+  end.
+
 is_always_string({{'.', _, [Module, Function]}, _, Args}) ->
   is_always_string(Module, Function, length(Args));
 is_always_string(Ast) ->
