@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("test_helper.exs", __DIR__)
 
 defmodule CodeTest do
@@ -95,12 +99,12 @@ defmodule CodeTest do
       sample = """
       defmodule CodeTest.UnknownRemoteCall do
         def perform do
-          UnkownModule.foo()
+          UnknownModule.foo()
         end
       end
       """
 
-      assert {_, [%{position: {3, 18}}]} =
+      assert {_, [%{position: {3, 19}}]} =
                Code.with_diagnostics(fn ->
                  quoted = Code.string_to_quoted!(sample, columns: true)
                  Code.eval_quoted(quoted, [])
@@ -388,7 +392,7 @@ defmodule CodeTest do
     Code.unrequire_files([fixture_path("code_sample.exs")])
   end
 
-  test "string_to_quoted!/2 errors take lines and columns into account" do
+  test "string_to_quoted!/2 errors take lines/columns/indentation into account" do
     assert_exception(
       SyntaxError,
       ["nofile:1:5:", "syntax error before:", "1 + * 3", "^"],
@@ -415,9 +419,9 @@ defmodule CodeTest do
 
     assert_exception(
       SyntaxError,
-      ["nofile:11:7:", "syntax error before:", "1 + * 3", "^"],
+      ["nofile:11:15:", "syntax error before:", "1 + * 3", "^"],
       fn ->
-        Code.string_to_quoted!(":ok\n1 + * 3", line: 10, column: 3)
+        Code.string_to_quoted!(":ok\n1 + * 3", line: 10, column: 3, indentation: 10)
       end
     )
   end
@@ -439,13 +443,166 @@ defmodule CodeTest do
     assert meta[:end_column] == 3
   end
 
-  test "compile source" do
-    assert __MODULE__.__info__(:compile)[:source] == String.to_charlist(__ENV__.file)
+  test "string_to_quoted with comments" do
+    assert Code.string_to_quoted_with_comments("""
+           # top
+           [
+             # before
+
+             # right-before
+             expr, # middle
+             # right-after
+
+             # after
+           ]
+           # bottom
+           """) ==
+             {
+               :ok,
+               [{:expr, [line: 6], nil}],
+               [
+                 %{
+                   column: 1,
+                   line: 1,
+                   next_eol_count: 1,
+                   previous_eol_count: 1,
+                   text: "# top"
+                 },
+                 %{
+                   column: 3,
+                   line: 3,
+                   next_eol_count: 2,
+                   previous_eol_count: 1,
+                   text: "# before"
+                 },
+                 %{
+                   column: 3,
+                   line: 5,
+                   next_eol_count: 1,
+                   previous_eol_count: 2,
+                   text: "# right-before"
+                 },
+                 %{
+                   column: 9,
+                   line: 6,
+                   next_eol_count: 1,
+                   previous_eol_count: 0,
+                   text: "# middle"
+                 },
+                 %{
+                   column: 3,
+                   line: 7,
+                   next_eol_count: 2,
+                   previous_eol_count: 1,
+                   text: "# right-after"
+                 },
+                 %{
+                   column: 3,
+                   line: 9,
+                   next_eol_count: 1,
+                   previous_eol_count: 2,
+                   text: "# after"
+                 },
+                 %{
+                   column: 1,
+                   line: 11,
+                   next_eol_count: 1,
+                   previous_eol_count: 1,
+                   text: "# bottom"
+                 }
+               ]
+             }
   end
 
-  test "compile info returned with source accessible through keyword module" do
-    compile = __MODULE__.__info__(:compile)
-    assert Keyword.get(compile, :source) != nil
+  test "string_to_quoted handles unescape errors properly" do
+    # Test invalid hex escape character
+    assert {:error, {meta, message, token}} = Code.string_to_quoted("a.'\\xg'")
+
+    assert meta[:line] == 1
+    assert meta[:column] == 3
+
+    assert message ==
+             "invalid hex escape character, expected \\xHH where H is a hexadecimal digit. Syntax error after: "
+
+    assert token == "\\x"
+
+    # Test invalid Unicode escape character
+    assert {:error, {meta2, message2, token2}} = Code.string_to_quoted("a.'\\ug'")
+
+    assert meta2[:line] == 1
+    assert meta2[:column] == 3
+
+    assert message2 ==
+             "invalid Unicode escape character, expected \\uHHHH or \\u{H*} where H is a hexadecimal digit. Syntax error after: "
+
+    assert token2 == "\\u"
+
+    # Test invalid Unicode code point (surrogate pair)
+    assert {:error, {meta3, message3, token3}} = Code.string_to_quoted("a.'\\u{D800}'")
+
+    assert meta3[:line] == 1
+    assert meta3[:column] == 3
+
+    assert message3 == "invalid or reserved Unicode code point \\u{D800}. Syntax error after: "
+    assert token3 == "\\u"
+
+    # Test Unicode code point beyond valid range
+    assert {:error, {meta4, message4, token4}} = Code.string_to_quoted("a.'\\u{110000}'")
+
+    assert meta4[:line] == 1
+    assert meta4[:column] == 3
+
+    assert message4 == "invalid or reserved Unicode code point \\u{110000}. Syntax error after: "
+    assert token4 == "\\u"
+  end
+
+  test "string_to_quoted returns error for invalid UTF-8 in strings" do
+    invalid_utf8_cases = [
+      # charlist
+      "'\\xFF'",
+      # charlist heredoc
+      "'''\n\\xFF\\\n'''"
+    ]
+
+    for code <- invalid_utf8_cases do
+      assert {:error, {_, message, _}} = Code.string_to_quoted(code)
+      assert message =~ "invalid encoding starting at <<255>>"
+    end
+  end
+
+  test "string_to_quoted returns error for invalid UTF-8 in quoted atoms and function calls" do
+    invalid_utf8_cases = [
+      # charlist
+      # ~S{'\xFF'},
+      # charlist heredoc
+      # ~s{'''\n\xFF\n'''},
+      # Quoted atom
+      ~S{:"\xFF"},
+      ~S{:'\xFF'},
+      # Quoted keyword identifier
+      ~S{["\xFF": 1]},
+      ~S{['\xFF': 1]},
+      # Quoted function call
+      ~S{foo."\xFF"()},
+      ~S{foo.'\xFF'()}
+    ]
+
+    for code <- invalid_utf8_cases do
+      assert {:error, {_, message, detail}} = Code.string_to_quoted(code)
+      assert message =~ "invalid encoding in atom: "
+      assert detail =~ "invalid encoding starting at <<255>>"
+
+      assert {:error, {_, message, detail}} =
+               Code.string_to_quoted(code, existing_atoms_only: true)
+
+      assert message =~ "invalid encoding in atom: "
+      assert detail =~ "invalid encoding starting at <<255>>"
+    end
+  end
+
+  @tag :requires_source
+  test "compile source" do
+    assert __MODULE__.__info__(:compile)[:source] == String.to_charlist(__ENV__.file)
   end
 
   describe "compile_string/1" do
@@ -475,7 +632,7 @@ defmodule CodeTest do
   end
 
   test "format_string/2 returns empty iodata for empty string" do
-    assert Code.format_string!("") == []
+    assert Code.format_string!("") == ""
   end
 
   test "ensure_loaded?/1" do
