@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../test_helper.exs", __DIR__)
 
 defmodule Kernel.ExpansionTarget do
@@ -114,6 +118,12 @@ defmodule Kernel.ExpansionTest do
       end)
     end
 
+    test "raises on multi-alias with non-atom base" do
+      assert_compile_error(~r"invalid alias: \"foo\"", fn ->
+        expand(quote(do: alias(foo.{Bar, Baz})))
+      end)
+    end
+
     test "invalid options" do
       assert_compile_error(~r"unsupported option :ops given to alias", fn ->
         expand(quote(do: alias(Foo, ops: 1)))
@@ -802,6 +812,15 @@ defmodule Kernel.ExpansionTest do
       end)
     end
 
+    test "in guards with macros" do
+      message =
+        ~r"you must require the moduleInteger before invoking macro Integer.is_even/1 inside a guard"
+
+      assert_compile_error(message, fn ->
+        expand(quote(do: fn arg when Integer.is_even(arg) -> arg end))
+      end)
+    end
+
     test "in guards with bitstrings" do
       message = ~r"cannot invoke remote function String.Chars.to_string/1 inside a guard"
 
@@ -1199,8 +1218,19 @@ defmodule Kernel.ExpansionTest do
 
     test "keeps position meta on & variables" do
       assert expand(Code.string_to_quoted!("& &1")) |> clean_meta([:counter]) ==
-               {:fn, [{:line, 1}],
-                [{:->, [{:line, 1}], [[{:capture, [line: 1], nil}], {:capture, [line: 1], nil}]}]}
+               {:fn, [capture: true, line: 1],
+                [
+                  {:->, [line: 1],
+                   [
+                     [{:capture, [capture: 1, line: 1], nil}],
+                     {:capture, [capture: 1, line: 1], nil}
+                   ]}
+                ]}
+    end
+
+    test "removes no_parens when expanding 0-arity capture to fn" do
+      assert expand(quote(do: &foo().bar/0)) ==
+               quote(do: fn -> foo().bar() end)
     end
 
     test "expands remotes" do
@@ -2382,6 +2412,15 @@ defmodule Kernel.ExpansionTest do
                |> clean_bit_modifiers()
     end
 
+    test "invalid match" do
+      assert_compile_error(
+        "a bitstring only accepts binaries, numbers, and variables inside a match",
+        fn ->
+          expand(quote(do: <<%{}>> = foo()))
+        end
+      )
+    end
+
     test "nested match" do
       assert expand(quote(do: <<foo = bar>>)) |> clean_meta([:alignment]) ==
                quote(do: <<foo = bar()::integer>>) |> clean_bit_modifiers()
@@ -2390,16 +2429,6 @@ defmodule Kernel.ExpansionTest do
              |> clean_meta([:alignment]) ==
                quote(do: <<45::integer, <<_::integer, _::binary>> = rest()::binary>>)
                |> clean_bit_modifiers()
-
-      message = ~r"cannot pattern match inside a bitstring that is already in match"
-
-      assert_compile_error(message, fn ->
-        expand(quote(do: <<bar = baz>> = foo()))
-      end)
-
-      assert_compile_error(message, fn ->
-        expand(quote(do: <<?-, <<_, _::binary>> = rest::binary>> = foo()))
-      end)
     end
 
     test "inlines binaries inside interpolation" do
@@ -2807,6 +2836,17 @@ defmodule Kernel.ExpansionTest do
       end)
     end
 
+    test "raises for variable used both in pattern and size" do
+      assert_compile_error(~r/undefined variable "foo"/, fn ->
+        code =
+          quote do
+            fn <<foo::size(foo)>> -> :ok end
+          end
+
+        expand(code, [])
+      end)
+    end
+
     test "raises for invalid unit" do
       message = ~r"unit in bitstring expects an integer as argument, got: :oops"
 
