@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../test_helper.exs", __DIR__)
 
 defmodule Kernel.WarningTest do
@@ -48,12 +52,6 @@ defmodule Kernel.WarningTest do
     end)
   end
 
-  defp capture_quoted(source) do
-    capture_err(fn ->
-      Code.string_to_quoted!(source, columns: true)
-    end)
-  end
-
   defp capture_compile(source) do
     capture_err(fn ->
       quoted = Code.string_to_quoted!(source, columns: true)
@@ -69,7 +67,7 @@ defmodule Kernel.WarningTest do
 
   test "warnings from macro" do
     assert_warn_eval(
-      ["demo:66\n", "key :dup will be overridden in map\n"],
+      ["demo:64\n", "key :dup will be overridden in map\n"],
       """
       import Kernel.WarningTest
       will_warn()
@@ -93,13 +91,6 @@ defmodule Kernel.WarningTest do
   end
 
   describe "unicode identifier security" do
-    test "prevents Restricted codepoints in identifiers" do
-      exception = assert_raise SyntaxError, fn -> Code.string_to_quoted!("_shibㅤ = 1") end
-
-      assert Exception.message(exception) =~
-               "unexpected token: \"ㅤ\" (column 6, code point U+3164)"
-    end
-
     test "warns on confusables" do
       assert_warn_quoted(
         ["nofile:1:6", "confusable identifier: 'a' looks like 'а' on line 1"],
@@ -139,57 +130,28 @@ defmodule Kernel.WarningTest do
                "confusable identifier: 'a' looks like 'а' on line 1"
     end
 
-    test "prevents unsafe script mixing in identifiers" do
-      exception =
-        assert_raise SyntaxError, fn ->
-          Code.string_to_quoted!("if аdmin_, do: :ok, else: :err")
-        end
+    test "warns on LTR-confusables" do
+      # warning outputs in byte order (vs bidi algo display order, uax9), mentions presence of rtl
+      assert_warn_quoted(
+        ["nofile:1:9", "'_1א' looks like '_א1'", "right-to-left characters"],
+        "_א1 and _1א"
+      )
 
-      assert Exception.message(exception) =~ "nofile:1:9:"
-      assert Exception.message(exception) =~ "invalid mixed-script identifier found: аdmin"
-
-      for s <- [
-            "\\u0430 а {Cyrillic}",
-            "\\u0064 d {Latin}",
-            "\\u006D m {Latin}",
-            "\\u0069 i {Latin}",
-            "\\u006E n {Latin}",
-            "\\u005F _"
-          ] do
-        assert Exception.message(exception) =~ s
-      end
-
-      # includes suggestion about what to change
-      assert Exception.message(exception) =~ """
-             Hint: You could write the above in a similar way that is accepted by Elixir:
-             """
-
-      assert Exception.message(exception) =~ """
-             "admin_" (code points 0x00061 0x00064 0x0006D 0x00069 0x0006E 0x0005F)
-             """
-
-      # a is in cyrillic
-      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("[аdmin: 1]") end
-      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("[{:аdmin, 1}]") end
-      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("quote do: аdmin(1)") end
-      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("рос_api = 1") end
-
-      # T is in cyrillic
-      assert_raise SyntaxError, ~r/mixed/, fn -> Code.string_to_quoted!("[Тシャツ: 1]") end
-    end
-
-    test "allows legitimate script mixing" do
-      # writing systems that legitimately mix multiple scripts, and Common chars like _
-      assert capture_eval("幻ㄒㄧㄤ = 1") == ""
-      assert capture_eval("幻ㄒㄧㄤ1 = 1") == ""
-      assert capture_eval("__सवव_1? = 1") == ""
-
-      # uts39 5.2 allowed 'highly restrictive' script mixing, like 't-shirt' in Jpan:
-      assert capture_quoted(":Tシャツ") == ""
-
-      # elixir's normalizations combine scriptsets of the 'from' and 'to' characters,
-      # ex: {Common} MICRO => {Greek} MU == {Common, Greek}; Common intersects w/all
-      assert capture_quoted("μs") == ""
+      assert_warn_quoted(
+        [
+          "'a_1א' includes right-to-left characters",
+          "\\u0061 a ltr",
+          "\\u005F _ neutral",
+          "\\u0031 1 weak_number",
+          "\\u05D0 א rtl",
+          "'a_א1' includes right-to-left characters:",
+          "\\u0061 a ltr",
+          "\\u005F _ neutral",
+          "\\u05D0 א rtl",
+          "\\u0031 1 weak_number"
+        ],
+        "a_א1 or a_1א"
+      )
     end
   end
 
@@ -232,7 +194,7 @@ defmodule Kernel.WarningTest do
       )
 
       assert_warn_eval(
-        ["nofile:1:3", "found quoted keyword \"foo\" but the quotes are not required"],
+        ["nofile:1:2", "found quoted keyword \"foo\" but the quotes are not required"],
         ~s/["foo": :bar]/
       )
 
@@ -243,6 +205,42 @@ defmodule Kernel.WarningTest do
     end
   end
 
+  describe "deprecated single quotes in atoms" do
+    test "warns for single quotes in atoms" do
+      assert_warn_eval(
+        [
+          "nofile:1:1",
+          "single quotes around atoms are deprecated. Use double quotes instead"
+        ],
+        ~s/:'a+b'/
+      )
+    end
+
+    test "warns twice for single and unnecessary atom quotes" do
+      assert_warn_eval(
+        [
+          "nofile:1:1",
+          "single quotes around atoms are deprecated. Use double quotes instead",
+          "nofile:1:1",
+          "found quoted atom \"ab\" but the quotes are not required"
+        ],
+        ~s/:'ab'/
+      )
+    end
+
+    test "warns twice for single and unnecessary call quotes" do
+      assert_warn_eval(
+        [
+          "nofile:1:9",
+          "single quotes around calls are deprecated. Use double quotes instead",
+          "nofile:1:9",
+          "found quoted call \"length\" but the quotes are not required"
+        ],
+        ~s/[Kernel.'length'([])]/
+      )
+    end
+  end
+
   test "warns on :: as atom" do
     assert_warn_eval(
       [
@@ -306,6 +304,20 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
+  test "duplicate pattern" do
+    output =
+      capture_eval("""
+      defmodule Sample do
+        var = quote(do: x)
+        def hello(unquote(var) = unquote(var)), do: unquote(var)
+      end
+      """)
+
+    assert output =~ "this pattern is matched against itself inside a match: x = x"
+  after
+    purge(Sample)
+  end
+
   test "unused compiler variable" do
     output =
       capture_eval("""
@@ -704,7 +716,10 @@ defmodule Kernel.WarningTest do
 
   test "unused default args" do
     assert_warn_eval(
-      ["nofile:3:8: ", "default values for the optional arguments in b/3 are never used"],
+      [
+        "nofile:3:8: ",
+        "default values for the optional arguments in the private function b/3 are never used"
+      ],
       ~S"""
       defmodule Sample1 do
         def a, do: b(1, 2, 3)
@@ -716,7 +731,7 @@ defmodule Kernel.WarningTest do
     assert_warn_eval(
       [
         "nofile:3:8: ",
-        "the default value for the last optional argument in b/3 is never used"
+        "the default value for the last optional argument in the private function b/3 is never used"
       ],
       ~S"""
       defmodule Sample2 do
@@ -729,7 +744,7 @@ defmodule Kernel.WarningTest do
     assert_warn_eval(
       [
         "nofile:3:8: ",
-        "the default values for the last 2 optional arguments in b/4 are never used"
+        "the default values for the last 2 optional arguments in the private function b/4 are never used"
       ],
       ~S"""
       defmodule Sample3 do
@@ -747,7 +762,10 @@ defmodule Kernel.WarningTest do
            """) == ""
 
     assert_warn_eval(
-      ["nofile:3:8: ", "the default value for the last optional argument in b/3 is never used"],
+      [
+        "nofile:3:8: ",
+        "the default value for the last optional argument in the private function b/3 is never used"
+      ],
       ~S"""
       defmodule Sample5 do
         def a, do: b(1, 2)
@@ -791,6 +809,19 @@ defmodule Kernel.WarningTest do
     )
   end
 
+  test "conditional import" do
+    assert capture_err(fn ->
+             defmodule KernelTest.ConditionalImport do
+               if false do
+                 import Map, only: [new: 0]
+                 def fun, do: new()
+               end
+             end
+           end) == ""
+  after
+    purge(Sample)
+  end
+
   test "unused import of one of the functions in :only" do
     assert_warn_compile(
       [
@@ -819,6 +850,22 @@ defmodule Kernel.WarningTest do
     )
   end
 
+  test "unused import inside dynamic module" do
+    import List, only: [flatten: 1], warn: false
+
+    assert capture_err(fn ->
+             defmodule Sample do
+               import String, only: [downcase: 1]
+
+               def world do
+                 flatten([1, 2, 3])
+               end
+             end
+           end) =~ "unused import String"
+  after
+    purge(Sample)
+  end
+
   def with(a, b, c), do: [a, b, c]
 
   test "import matches special form" do
@@ -854,6 +901,17 @@ defmodule Kernel.WarningTest do
     )
   end
 
+  test "conditional alias" do
+    assert capture_err(fn ->
+             defmodule KernelTest.ConditionaAlias do
+               if false do
+                 alias Map, as: M
+                 def fun, do: M.new()
+               end
+             end
+           end) == ""
+  end
+
   test "unused alias" do
     assert_warn_compile(
       ["nofile:2:3", "unused alias List"],
@@ -883,22 +941,6 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "unused inside dynamic module" do
-    import List, only: [flatten: 1], warn: false
-
-    assert capture_err(fn ->
-             defmodule Sample do
-               import String, only: [downcase: 1]
-
-               def world do
-                 flatten([1, 2, 3])
-               end
-             end
-           end) =~ "unused import String"
-  after
-    purge(Sample)
-  end
-
   test "duplicate map keys" do
     assert_warn_eval(
       [
@@ -923,25 +965,6 @@ defmodule Kernel.WarningTest do
     assert map_size(%{System.unique_integer() => 1, System.unique_integer() => 2}) == 2
   end
 
-  test "unused guard" do
-    assert_warn_eval(
-      ["nofile:5\n", "this check/guard will always yield the same result"],
-      """
-      defmodule Sample do
-        def atom_case do
-          v = "bc"
-          case v do
-            _ when is_atom(v) -> :ok
-            _ -> :fail
-          end
-        end
-      end
-      """
-    )
-  after
-    purge(Sample)
-  end
-
   test "length(list) == 0 in guard" do
     assert_warn_eval(
       [
@@ -1059,7 +1082,7 @@ defmodule Kernel.WarningTest do
   test "clause not match" do
     assert_warn_eval(
       [
-        "nofile:3\n",
+        "nofile:3:7\n",
         ~r"this clause( for hello/0)? cannot match because a previous clause at line 2 always matches"
       ],
       """
@@ -1110,7 +1133,7 @@ defmodule Kernel.WarningTest do
     message = "def hello/1 has multiple clauses and also declares default values"
 
     assert_warn_eval(
-      ["nofile:3\n", message],
+      ["nofile:3:7\n", message, "the previous clause is defined on line 2"],
       ~S"""
       defmodule Sample1 do
         def hello(arg), do: arg
@@ -1120,7 +1143,7 @@ defmodule Kernel.WarningTest do
     )
 
     assert_warn_eval(
-      ["nofile:3:7\n", message],
+      ["nofile:3:7\n", message, "the previous clause is defined on line 2"],
       ~S"""
       defmodule Sample2 do
         def hello(_arg)
@@ -1134,7 +1157,11 @@ defmodule Kernel.WarningTest do
 
   test "clauses with default should use header" do
     assert_warn_eval(
-      ["nofile:3\n", "def hello/1 has multiple clauses and also declares default values"],
+      [
+        "nofile:3:7\n",
+        "def hello/1 has multiple clauses and also declares default values",
+        "the previous clause is defined on line 2"
+      ],
       ~S"""
       defmodule Sample do
         def hello(arg \\ 0), do: arg
@@ -1254,22 +1281,9 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "in guard empty list" do
-    assert_warn_eval(
-      ["nofile:2\n", "this check/guard will always yield the same result"],
-      """
-      defmodule Sample do
-        def a(x) when x in [], do: x
-      end
-      """
-    )
-  after
-    purge(Sample)
-  end
-
   test "no effect operator" do
     assert_warn_eval(
-      ["nofile:3\n", "use of operator != has no effect"],
+      ["nofile:3:7\n", "use of operator != has no effect"],
       """
       defmodule Sample do
         def a(x) do
@@ -1283,28 +1297,6 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "eval failure warning" do
-    assert_warn_eval(
-      ["nofile:2\n", "the call to Atom.to_string/1 will fail with ArgumentError"],
-      """
-      defmodule Sample1 do
-        def foo, do: Atom.to_string "abc"
-      end
-      """
-    )
-
-    assert_warn_eval(
-      ["nofile:2\n", "the call to +/2 will fail with ArithmeticError"],
-      """
-      defmodule Sample2 do
-        def foo, do: 1 + nil
-      end
-      """
-    )
-  after
-    purge([Sample1, Sample2])
-  end
-
   test "undefined function for behaviour" do
     assert_warn_eval(
       [
@@ -1595,16 +1587,11 @@ defmodule Kernel.WarningTest do
         @doc "Something"
         @doc "Another"
         def foo, do: :ok
-
-        Module.eval_quoted(__MODULE__, quote(do: @doc false))
-        @doc "Doc"
-        def bar, do: :ok
       end
       """)
 
     assert output =~ "redefining @doc attribute previously set at line 2"
     assert output =~ "nofile:3: Sample (module)"
-    refute output =~ "nofile:7: "
   after
     purge(Sample)
   end
@@ -1835,6 +1822,20 @@ defmodule Kernel.WarningTest do
       purge([Sample1, Sample2, Sample3])
     end
 
+    test "invalid fun" do
+      assert_warn_eval(
+        [
+          "nofile:2: ",
+          "fun/1 is not valid in typespecs. Either specify fun() or use (... -> return) instead"
+        ],
+        """
+        defmodule InvalidFunType do
+          @type my_type :: fun(integer())
+        end
+        """
+      )
+    end
+
     test "invalid type annotations" do
       assert_warn_eval(
         [
@@ -1953,7 +1954,9 @@ defmodule Kernel.WarningTest do
   end
 
   test "variable is being expanded to function call (on_undefined_variable: warn)" do
-    Code.put_compiler_option(:on_undefined_variable, :warn)
+    capture_io(:stderr, fn ->
+      Code.put_compiler_option(:on_undefined_variable, :warn)
+    end)
 
     output =
       capture_eval("""
@@ -2074,7 +2077,7 @@ defmodule Kernel.WarningTest do
   test "defguard overriding defmacro" do
     assert_warn_eval(
       [
-        "nofile:3\n",
+        "nofile:3:12\n",
         ~r"this clause( for foo/1)? cannot match because a previous clause at line 2 always matches"
       ],
       """
@@ -2091,7 +2094,7 @@ defmodule Kernel.WarningTest do
   test "defmacro overriding defguard" do
     assert_warn_eval(
       [
-        "nofile:3\n",
+        "nofile:3:12\n",
         ~r"this clause( for foo/1)? cannot match because a previous clause at line 2 always matches"
       ],
       """
@@ -2136,29 +2139,6 @@ defmodule Kernel.WarningTest do
     purge(Sample)
   end
 
-  test "nested comparison operators" do
-    message =
-      capture_compile("""
-       1 < 3 < 5
-      """)
-
-    assert message =~ "nofile:1:8\n"
-    assert message =~ "Elixir does not support nested comparisons"
-    assert message =~ "1 < 3 < 5"
-
-    message =
-      capture_compile("""
-        x = 5
-        y = 7
-        1 < x < y < 10
-      """)
-
-    assert message =~ "Elixir does not support nested comparisons"
-    assert message =~ "1 < x < y < 10"
-    assert message =~ "nofile:3:9\n"
-    assert message =~ "nofile:3:13\n"
-  end
-
   test "def warns if only clause is else" do
     assert_warn_compile(
       ["nofile:2:7\n", "\"else\" shouldn't be used as the only clause in \"def\""],
@@ -2214,15 +2194,15 @@ defmodule Kernel.WarningTest do
 
   test "defstruct warns with duplicate keys" do
     assert_warn_eval(
-      ["nofile:2: TestMod", "duplicate key :foo found in struct"],
+      ["nofile:2: Sample", "duplicate key :foo found in struct"],
       """
-      defmodule TestMod do
+      defmodule Sample do
         defstruct [:foo, :bar, foo: 1]
       end
       """
     )
   after
-    purge(TestMod)
+    purge(Sample)
   end
 
   test "deprecate nullary remote zero-arity capture with parens" do
@@ -2242,6 +2222,25 @@ defmodule Kernel.WarningTest do
     )
   end
 
+  test "deprecate &module.fun/arity captures with complex expressions as modules" do
+    assert_warn_eval(
+      [
+        "nofile:2:",
+        """
+        expected the module in &module.fun/arity to expand to a variable or an atom, got: hd(modules)
+        You can either compute the module name outside of & or convert it to a regular anonymous function.
+        """
+      ],
+      """
+      defmodule Sample do
+        def foo(modules), do: &hd(modules).module_info/0
+      end
+      """
+    )
+  after
+    purge(Sample)
+  end
+
   defp assert_compile_error(messages, string) do
     captured =
       capture_err(fn ->
@@ -2261,7 +2260,7 @@ defmodule Kernel.WarningTest do
   end
 
   defp purge(module) when is_atom(module) do
-    :code.delete(module)
     :code.purge(module)
+    :code.delete(module)
   end
 end
