@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("test_helper.exs", __DIR__)
 
 defmodule MapTest do
@@ -221,11 +225,11 @@ defmodule MapTest do
     assert Map.replace!(map, :b, 10) == %{c: 3, b: 10, a: 1}
     assert Map.replace!(map, :a, 1) == map
 
-    assert_raise KeyError, ~r/key :x not found in: %{.*a: 1.*}/, fn ->
+    assert_raise KeyError, ~r/key :x not found in:\n\n    %{.*a: 1.*}/, fn ->
       Map.replace!(map, :x, 10)
     end
 
-    assert_raise KeyError, "key :x not found in: %{}", fn ->
+    assert_raise KeyError, "key :x not found in:\n\n    %{}\n", fn ->
       Map.replace!(%{}, :x, 10)
     end
   end
@@ -283,12 +287,8 @@ defmodule MapTest do
 
   test "structs" do
     assert %ExternalUser{} == %{__struct__: ExternalUser, name: "john", age: 27}
-
     assert %ExternalUser{name: "meg"} == %{__struct__: ExternalUser, name: "meg", age: 27}
 
-    user = %ExternalUser{}
-    assert %ExternalUser{user | name: "meg"} == %{__struct__: ExternalUser, name: "meg", age: 27}
-
     %ExternalUser{name: name} = %ExternalUser{}
     assert name == "john"
   end
@@ -401,7 +401,7 @@ defmodule MapTest do
     end
 
     info = Macro.struct_info!(WithBitstring, __ENV__)
-    assert info == [%{default: <<255, 127::7>>, field: :bitstring}]
+    assert info == [%{default: <<255, 127::7>>, field: :bitstring, required: false}]
   end
 
   test "defstruct can only be used once in a module" do
