@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("test_helper.exs", __DIR__)
 
 defmodule ModuleTest.ToBeUsed do
@@ -35,7 +39,7 @@ end
 
 defmodule ModuleTest.ToUse do
   # Moving the next line around can make tests fail
-  38 = __ENV__.line
+  42 = __ENV__.line
   var = 1
   # Not available in callbacks
   _ = var
@@ -54,13 +58,6 @@ defmodule ModuleTest do
   @register_example :it_works
   @register_example :still_works
 
-  contents =
-    quote do
-      def eval_quoted_info, do: {__MODULE__, __ENV__.file, __ENV__.line}
-    end
-
-  Module.eval_quoted(__MODULE__, contents, [], file: "sample.ex", line: 13)
-
   defp purge(module) do
     :code.purge(module)
     :code.delete(module)
@@ -126,34 +123,12 @@ defmodule ModuleTest do
     assert :code.which(__MODULE__) == ~c""
   end
 
-  ## Eval
-
-  test "executes eval_quoted definitions" do
-    assert eval_quoted_info() == {ModuleTest, "sample.ex", 13}
-  end
-
-  test "resets last definition information on eval" do
-    # This should not emit any warning
-    defmodule LastDefinition do
-      def foo(0), do: 0
-
-      Module.eval_quoted(
-        __ENV__,
-        quote do
-          def bar, do: :ok
-        end
-      )
-
-      def foo(1), do: 1
-    end
-  end
+  ## Callbacks
 
   test "retrieves line from use callsite" do
-    assert ModuleTest.ToUse.line() == 43
+    assert ModuleTest.ToUse.line() == 47
   end
 
-  ## Callbacks
-
   test "executes custom before_compile callback" do
     assert ModuleTest.ToUse.callback_value(true) == true
     assert ModuleTest.ToUse.callback_value(false) == false
@@ -228,14 +203,6 @@ defmodule ModuleTest do
     assert @other_attribute == [3, 2, 1]
   end
 
-  test "@compile autoload attribute" do
-    defmodule NoAutoload do
-      @compile {:autoload, false}
-    end
-
-    refute Code.loaded?(NoAutoload)
-  end
-
   ## Naming
 
   test "concat" do
@@ -446,6 +413,17 @@ defmodule ModuleTest do
     assert map.module == ModuleCreateDebugInfo
   end
 
+  test "uses the debug_info chunk when explicitly set to true" do
+    {:module, ModuleCreateDebugInfoTrue, binary, _} =
+      Module.create(ModuleCreateDebugInfoTrue, quote(do: @compile({:debug_info, true})), __ENV__)
+
+    {:ok, {_, [debug_info: {:debug_info_v1, backend, data}]}} =
+      :beam_lib.chunks(binary, [:debug_info])
+
+    {:ok, map} = backend.debug_info(:elixir_v1, ModuleCreateDebugInfoTrue, data, [])
+    assert map.module == ModuleCreateDebugInfoTrue
+  end
+
   test "uses the debug_info chunk even if debug_info is set to false" do
     {:module, ModuleCreateNoDebugInfo, binary, _} =
       Module.create(ModuleCreateNoDebugInfo, quote(do: @compile({:debug_info, false})), __ENV__)
@@ -457,9 +435,18 @@ defmodule ModuleTest do
   end
 
   test "compiles to core" do
-    {:ok, {Atom, [{~c"Dbgi", dbgi}]}} = Atom |> :code.which() |> :beam_lib.chunks([~c"Dbgi"])
+    import PathHelpers
+
+    write_beam(
+      defmodule ExampleModule do
+      end
+    )
+
+    {:ok, {ExampleModule, [{~c"Dbgi", dbgi}]}} =
+      ExampleModule |> :code.which() |> :beam_lib.chunks([~c"Dbgi"])
+
     {:debug_info_v1, backend, data} = :erlang.binary_to_term(dbgi)
-    {:ok, core} = backend.debug_info(:core_v1, Atom, data, [])
+    {:ok, core} = backend.debug_info(:core_v1, ExampleModule, data, [])
     assert is_tuple(core)
   end
 
