@@ -1,25 +1,6 @@
-defmodule ExUnit.DuplicateTestError do
-  @moduledoc """
-  Exception raised to indicate two or more tests with the same name.
-  """
-
-  @typedoc since: "1.16.0"
-  @type t :: %__MODULE__{message: String.t()}
-
-  defexception [:message]
-end
-
-defmodule ExUnit.DuplicateDescribeError do
-  @moduledoc """
-  Exception raised to indicate two or more `describe` blocks with
-  the same name.
-  """
-
-  @typedoc since: "1.16.0"
-  @type t :: %__MODULE__{message: String.t()}
-
-  defexception [:message]
-end
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
 
 defmodule ExUnit.Case do
   @moduledoc """
@@ -31,13 +12,24 @@ defmodule ExUnit.Case do
   When used, it accepts the following options:
 
     * `:async` - configures tests in this module to run concurrently with
-      tests in other modules. Tests in the same module never run concurrently.
+      tests in other modules. Tests in the same module never run concurrently
+      (with the exception of tests run via the `:parameterize` option - see below).
       It should be enabled only if tests do not change any global state.
       Defaults to `false`.
 
+    * `:group` (since v1.18.0) - configures the **group** this module belongs to.
+      Tests in the same group never run concurrently. Tests from different
+      groups (or with no groups) can run concurrently when `async: true`
+      is given. By default, this module belongs to no group (defaults to `nil`).
+
     * `:register` - when `false`, does not register this module within
       ExUnit server. This means the module won't run when ExUnit suite runs.
 
+    * `:parameterize` (since v1.18.0) - a list of maps to parameterize tests.
+      If both `:async` and `:parameterize` are given, the different parameters
+      run concurrently. See the "Parameterized tests" section below for more
+      information.
+
   > #### `use ExUnit.Case` {: .info}
   >
   > When you `use ExUnit.Case`, it will import the functionality
@@ -160,30 +152,38 @@ defmodule ExUnit.Case do
   The following tags are set automatically by ExUnit and are
   therefore reserved:
 
-    * `:module` - the module on which the test was defined
+    * `:async` - if the test case is in async mode
 
     * `:file` - the file on which the test was defined
 
     * `:line` - the line on which the test was defined
 
-    * `:test` - the test name
-
-    * `:async` - if the test case is in async mode
+    * `:module` - the module on which the test was defined
 
     * `:registered` - used for `ExUnit.Case.register_attribute/3` values
 
-    * `:describe` - the describe block the test belongs to
+    * `:test` - the test name
 
-    * `:describe_line` - the line the describe block begins on
+    * `:test_group` - the group the test belongs to
+
+    * `:test_pid` - the PID of the testing process
+
+    * `:test_type` - the test type used when printing test results. It can also
+      be used for filtering. It is set by ExUnit to `:test`, `:doctest`, or
+      the equivalent type given to `register_test/6`
+
+    * `:describe` - the describe block the test belongs to (if in a describe)
+
+    * `:describe_line` - the line the describe block begins on (if in a describe).
+      It can be used to run all tests in the describe block given by line
 
     * `:doctest` - the module or the file being doctested (if a doctest)
 
-    * `:doctest_line` - the line the doctest was defined (if a doctest)
+    * `:doctest_data` - additional metadata about doctests stored in a map,
+      such as the `:end_line`, available for reflection purposes (if a doctest)
 
-    * `:doctest_data` - additional metadata about doctests (if a doctest)
-
-    * `:test_type` - the test type used when printing test results.
-      It is set by ExUnit to `:test`, `:doctest` and so on, but is customizable.
+    * `:doctest_line` - the line the doctest was defined (if a doctest).
+      It can be used to run a doctest defined at the given source line
 
   The following tags customize how tests behave:
 
@@ -191,11 +191,46 @@ defmodule ExUnit.Case do
 
     * `:skip` - skips the test with the given reason
 
-    * `:timeout` - customizes the test timeout in milliseconds (defaults to 60000).
+    * `:timeout` - customizes the test timeout in milliseconds (defaults to `60_000`).
       Accepts `:infinity` as a timeout value.
 
     * `:tmp_dir` - (since v1.11.0) see the "Tmp Dir" section below
 
+  ## Parameterized tests
+
+  Sometimes you want to run the same tests but with different parameters.
+  In ExUnit, it is possible to do so by passing a `:parameterize` key to
+  `ExUnit.Case`. The value must be a list of maps which will be the
+  parameters merged into the test context.
+
+  For example, Elixir has a module called `Registry`, which can have type
+  `:unique` or `:duplicate`, and can control its concurrency factor using
+  the `:partitions` option. If you have a number of tests that *behave the
+  same* across all of those values, you can parameterize those tests with:
+
+      use ExUnit.Case,
+        async: true,
+        parameterize:
+          for(kind <- [:unique, :duplicate],
+              partitions <- [1, 8],
+              do: %{kind: kind, partitions: partitions})
+
+  Then, in your tests, you can access the parameters as part of the context:
+
+      test "starts a registry", %{kind: kind, partitions: partitions} do
+        ...
+      end
+
+  Use parameterized tests with care:
+
+    * Although parameterized tests run concurrently when `async: true` is also given,
+      abuse of parameterized tests may make your test suite slower
+
+    * If you use parameterized tests and then find yourself adding conditionals
+      in your tests to deal with different parameters, then parameterized tests
+      may be the wrong solution to your problem. Consider creating separated
+      tests and sharing logic between them using regular functions
+
   ## Filters
 
   Tags can also be used to identify specific tests, which can then
@@ -245,7 +280,7 @@ defmodule ExUnit.Case do
   in them (or between tests) are never captured. If you want to suppress these
   messages as well, remove the console backend globally by setting:
 
-      config :logger, backends: []
+      config :logger, :default_handler, false
 
   ## Tmp Dir
 
@@ -273,18 +308,52 @@ defmodule ExUnit.Case do
 
   As with other tags, `:tmp_dir` can also be set as `@moduletag` and
   `@describetag`.
+
+  ## Process Architecture
+
+  An ExUnit test case uses several processes when it runs. These are illustrated below.
+
+  ```mermaid
+  sequenceDiagram
+    participant runner as ExUnit Case
+    runner->>runner: Run setup_all callbacks
+
+    loop Each test
+      create participant test as Test Process
+      runner->>test: Spawn
+      test->>test: Run setup callbacks
+      test->>test: Run test
+      destroy test
+      test-xrunner: Exits
+      runner->>runner: Run on_exit callbacks
+    end
+  ```
+
+    1. First, all `ExUnit.Callbacks.setup_all/1` callbacks run in a single process, sequentially,
+       in the order they were defined.
+
+    2. Then, a new process is spawned for the test itself. In this process, first all
+       `ExUnit.Callbacks.setup/1` callbacks run, in the order they were defined. Then,
+       the test itself is executed.
+
+    3. After the test exits, a new process is spawned to run all `ExUnit.Callbacks.on_exit/2`,
+       in the reverse order they were defined.
+
   """
 
   @type env :: module() | Macro.Env.t()
+
+  @type register_attribute_opts :: [
+          accumulate: boolean(),
+          persist: boolean()
+        ]
   @compile {:no_warn_undefined, [IEx.Pry]}
   @reserved [:module, :file, :line, :test, :async, :registered, :describe]
 
   @doc false
   defmacro __using__(opts) do
     quote do
-      unless ExUnit.Case.__register__(__MODULE__, unquote(opts)) do
-        use ExUnit.Callbacks
-      end
+      ExUnit.Case.__register__(__MODULE__, unquote(opts))
 
       import ExUnit.Callbacks
       import ExUnit.Assertions
@@ -293,17 +362,27 @@ defmodule ExUnit.Case do
     end
   end
 
+  @keys [:async, :group, :parameterize, :register]
+
+  @doc false
+  def __keys__(opts), do: Keyword.take(opts, @keys)
+
   @doc false
   def __register__(module, opts) do
-    unless Keyword.keyword?(opts) do
+    if not Keyword.keyword?(opts) do
       raise ArgumentError,
             ~s(the argument passed to "use ExUnit.Case" must be a list of options, ) <>
               ~s(got: #{inspect(opts)})
     end
 
-    registered? = Module.has_attribute?(module, :ex_unit_tests)
+    {register?, opts} = Keyword.pop(opts, :register, true)
+    {next_opts, opts} = Keyword.split(opts, @keys)
 
-    unless registered? do
+    if opts != [] do
+      IO.warn("unknown options given to ExUnit.Case: #{inspect(opts)}")
+    end
+
+    if not Module.has_attribute?(module, :ex_unit_tests) do
       tag_check = Enum.any?([:moduletag, :describetag, :tag], &Module.has_attribute?(module, &1))
 
       if tag_check do
@@ -322,24 +401,17 @@ defmodule ExUnit.Case do
 
       Enum.each(accumulate_attributes, &Module.register_attribute(module, &1, accumulate: true))
 
-      persisted_attributes = [:ex_unit_async]
-
-      Enum.each(persisted_attributes, &Module.register_attribute(module, &1, persist: true))
-
-      if Keyword.get(opts, :register, true) do
+      if register? do
         Module.put_attribute(module, :after_compile, ExUnit.Case)
       end
 
+      ExUnit.Callbacks.__register__(module)
       Module.put_attribute(module, :before_compile, ExUnit.Case)
     end
 
-    async? = opts[:async]
-
-    if is_boolean(async?) or not registered? do
-      Module.put_attribute(module, :ex_unit_async, async? || false)
-    end
-
-    registered?
+    past_opts = Module.get_attribute(module, :ex_unit_module, [])
+    Module.put_attribute(module, :ex_unit_module, Keyword.merge(past_opts, next_opts))
+    :ok
   end
 
   @doc """
@@ -357,7 +429,7 @@ defmodule ExUnit.Case do
 
   """
   defmacro test(message, var \\ quote(do: _), contents) do
-    unless is_tuple(var) do
+    if not is_tuple(var) do
       IO.warn(
         "test context is always a map. The pattern " <>
           "#{inspect(Macro.to_string(var))} will never match",
@@ -521,58 +593,80 @@ defmodule ExUnit.Case do
   end
 
   @doc false
-  defmacro __before_compile__(env) do
+  defmacro __before_compile__(%{module: module}) do
     tests =
-      env.module
+      module
       |> Module.get_attribute(:ex_unit_tests)
       |> Enum.reverse()
       |> Macro.escape()
 
-    moduletag = Module.get_attribute(env.module, :moduletag)
+    moduletag = Module.get_attribute(module, :moduletag)
 
     tags =
       moduletag
       |> normalize_tags()
       |> validate_tags()
       |> Map.new()
-      |> Map.merge(%{module: env.module, case: env.module})
+
+    opts = Module.get_attribute(module, :ex_unit_module, [])
+    async? = Keyword.get(opts, :async, false)
+    group = Keyword.get(opts, :group, nil)
+    parameterize = Keyword.get(opts, :parameterize, nil)
+
+    if not is_boolean(async?) do
+      raise ArgumentError, ":async must be a boolean, got: #{inspect(async?)}"
+    end
+
+    if not (parameterize == nil or (is_list(parameterize) and Enum.all?(parameterize, &is_map/1))) do
+      raise ArgumentError, ":parameterize must be a list of maps, got: #{inspect(parameterize)}"
+    end
+
+    {setup_all?, callbacks} = ExUnit.Callbacks.__callbacks__(module)
 
     quote do
+      unquote(callbacks)
+
       def __ex_unit__ do
         %ExUnit.TestModule{
           file: __ENV__.file,
           name: __MODULE__,
+          setup_all?: unquote(setup_all?),
           tags: unquote(Macro.escape(tags)),
           tests: unquote(tests)
         }
       end
+
+      def __ex_unit__(:config) do
+        %{
+          async?: unquote(async?),
+          group: unquote(Macro.escape(group)),
+          parameterize: unquote(Macro.escape(parameterize))
+        }
+      end
     end
   end
 
   @doc false
   def __after_compile__(%{module: module}, _) do
     cond do
-      Process.whereis(ExUnit.Server) == nil ->
-        unless Code.can_await_module_compilation?() do
-          raise "cannot use ExUnit.Case without starting the ExUnit application, " <>
-                  "please call ExUnit.start() or explicitly start the :ex_unit app"
-        end
+      Process.whereis(ExUnit.Server) ->
+        ExUnit.Server.add_module(module, module.__ex_unit__(:config))
 
-      Module.get_attribute(module, :ex_unit_async) ->
-        ExUnit.Server.add_async_module(module)
+      Code.can_await_module_compilation?() ->
+        :ok
 
       true ->
-        ExUnit.Server.add_sync_module(module)
+        raise "cannot use ExUnit.Case without starting the ExUnit application, " <>
+                "please call ExUnit.start() or explicitly start the :ex_unit app"
     end
   end
 
   @doc """
-  Registers a function to run as part of this case.
+  Registers a function to run as a test for this module.
 
-  This is used by third-party projects, like QuickCheck, to
-  implement macros like `property/3` that works like `test`
-  but instead defines a property. See `test/3` implementation
-  for an example of invoking this function.
+  This is used by third-party projects to implement macros like
+  `property/3` that works like `test` but instead defines a property.
+  See `test/3` implementation for an example of invoking this function.
 
   The test type will be converted to a string and pluralized for
   display. You can use `ExUnit.plural_rule/2` to set a custom
@@ -580,7 +674,7 @@ defmodule ExUnit.Case do
   """
   @doc since: "1.11.0"
   def register_test(mod, file, line, test_type, name, tags) do
-    unless Module.has_attribute?(mod, :ex_unit_tests) do
+    if not Module.has_attribute?(mod, :ex_unit_tests) do
       raise "cannot define #{test_type}. Please make sure you have invoked " <>
               "\"use ExUnit.Case\" in the current module"
     end
@@ -600,7 +694,6 @@ defmodule ExUnit.Case do
 
     moduletag = Module.get_attribute(mod, :moduletag)
     tag = Module.delete_attribute(mod, :tag)
-    async = Module.get_attribute(mod, :ex_unit_async)
 
     {name, describe, describe_line, describetag} =
       case Module.get_attribute(mod, :ex_unit_describe) do
@@ -614,7 +707,7 @@ defmodule ExUnit.Case do
       end
 
     if Module.defines?(mod, {name, 1}) do
-      raise ExUnit.DuplicateTestError, ~s("#{name}" is already defined in #{inspect(mod)})
+      raise ArgumentError, ~s("#{name}" is already defined in #{inspect(mod)})
     end
 
     tags =
@@ -625,7 +718,6 @@ defmodule ExUnit.Case do
         line: line,
         file: file,
         registered: registered,
-        async: async,
         describe: describe,
         describe_line: describe_line,
         test_type: test_type
@@ -683,7 +775,7 @@ defmodule ExUnit.Case do
 
   """
   @doc since: "1.3.0"
-  @spec register_attribute(env, atom, keyword) :: :ok
+  @spec register_attribute(env, atom, register_attribute_opts) :: :ok
   def register_attribute(env, name, opts \\ [])
   def register_attribute(%{module: mod}, name, opts), do: register_attribute(mod, name, opts)
 
@@ -725,7 +817,7 @@ defmodule ExUnit.Case do
 
   """
   @doc since: "1.10.0"
-  @spec register_describe_attribute(env, atom, keyword) :: :ok
+  @spec register_describe_attribute(env, atom, register_attribute_opts) :: :ok
   def register_describe_attribute(env, name, opts \\ [])
 
   def register_describe_attribute(%{module: mod}, name, opts) do
@@ -765,7 +857,7 @@ defmodule ExUnit.Case do
 
   """
   @doc since: "1.10.0"
-  @spec register_module_attribute(env, atom, keyword) :: :ok
+  @spec register_module_attribute(env, atom, register_attribute_opts) :: :ok
   def register_module_attribute(env, name, opts \\ [])
 
   def register_module_attribute(%{module: mod}, name, opts) do
@@ -826,7 +918,7 @@ defmodule ExUnit.Case do
       raise "cannot set tag #{inspect(tag)} because it is reserved by ExUnit"
     end
 
-    unless is_atom(tags[:test_type]) do
+    if not is_atom(tags[:test_type]) do
       raise("value for tag \":test_type\" must be an atom")
     end
 
@@ -848,9 +940,19 @@ defmodule ExUnit.Case do
 
   defp normalize_tags(tags) do
     Enum.reduce(Enum.reverse(tags), %{}, fn
-      {key, value}, acc -> Map.put(acc, key, value)
-      tag, acc when is_atom(tag) -> Map.put(acc, tag, true)
-      tag, acc when is_list(tag) -> Enum.into(tag, acc)
+      {key, value}, acc ->
+        Map.put(acc, key, value)
+
+      tag, acc when is_atom(tag) ->
+        Map.put(acc, tag, true)
+
+      tag, acc ->
+        if Keyword.keyword?(tag) do
+          Enum.into(tag, acc)
+        else
+          raise "an invalid value for a tag was used. " <>
+                  "Expected an atom or a keyword list, got: #{inspect(tag)}"
+        end
     end)
   end
 end
