@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule ExUnit.Runner do
   @moduledoc false
 
@@ -125,7 +129,7 @@ defmodule ExUnit.Runner do
 
         # Run all sync modules directly
         for pair <- sync_modules do
-          running = spawn_modules(config, [[pair]], false, %{})
+          running = spawn_modules(config, [{nil, [pair]}], false, %{})
           running != %{} and wait_until_available(config, running)
         end
 
@@ -161,18 +165,23 @@ defmodule ExUnit.Runner do
     running
   end
 
-  defp spawn_modules(config, [[_ | _] = modules | groups], async?, running) do
-    if max_failures_reached?(config) do
-      running
-    else
-      {pid, ref} =
-        spawn_monitor(fn ->
-          Enum.each(modules, fn {module, params} ->
-            run_module(config, module, async?, params)
-          end)
-        end)
+  defp spawn_modules(config, [{group, modules} | groups], async?, running) do
+    cond do
+      max_failures_reached?(config) ->
+        running
 
-      spawn_modules(config, groups, async?, Map.put(running, ref, pid))
+      modules == [] ->
+        spawn_modules(config, groups, async?, running)
+
+      true ->
+        {pid, ref} =
+          spawn_monitor(fn ->
+            Enum.each(modules, fn {module, params} ->
+              run_module(config, module, async?, group, params)
+            end)
+          end)
+
+        spawn_modules(config, groups, async?, Map.put(running, ref, pid))
     end
   end
 
@@ -227,12 +236,13 @@ defmodule ExUnit.Runner do
 
   ## Running modules
 
-  defp run_module(config, module, async?, params) do
+  defp run_module(config, module, async?, group, params) do
     test_module = %{module.__ex_unit__() | parameters: params}
     EM.module_started(config.manager, test_module)
 
     # Prepare tests, selecting which ones should be run or skipped
-    {to_run_tests, excluded_and_skipped_tests} = prepare_tests(config, async?, test_module.tests)
+    {to_run_tests, excluded_and_skipped_tests} =
+      prepare_tests(config, async?, group, test_module.tests)
 
     for excluded_or_skipped_test <- excluded_and_skipped_tests do
       EM.test_started(config.manager, excluded_or_skipped_test)
@@ -267,7 +277,7 @@ defmodule ExUnit.Runner do
     end
   end
 
-  defp prepare_tests(config, async?, tests) do
+  defp prepare_tests(config, async?, group, tests) do
     tests = shuffle(config, tests)
     include = config.include
     exclude = config.exclude
@@ -276,7 +286,13 @@ defmodule ExUnit.Runner do
     {to_run, to_skip} =
       for test <- tests, include_test?(test_ids, test), reduce: {[], []} do
         {to_run, to_skip} ->
-          tags = Map.merge(test.tags, %{test: test.name, module: test.module, async: async?})
+          tags =
+            Map.merge(test.tags, %{
+              test: test.name,
+              module: test.module,
+              async: async?,
+              test_group: group
+            })
 
           case ExUnit.Filters.eval(include, exclude, tags, tests) do
             :ok -> {[%{test | tags: tags} | to_run], to_skip}
@@ -424,6 +440,7 @@ defmodule ExUnit.Runner do
          context
        ) do
     spawn_monitor(fn ->
+      Process.set_label({test.case, test.name})
       ExUnit.OnExitHandler.register(self())
       generate_test_seed(seed, test, rand_algorithm)
       context = context |> Map.merge(test.tags) |> Map.put(:test_pid, self())
