@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule IEx do
   @moduledoc ~S"""
   Elixir's interactive shell.
@@ -26,10 +30,6 @@ defmodule IEx do
   of `__foo__`.
 
   Autocomplete is available by default on Windows shells from Erlang/OTP 26.
-  In earlier versions, you may need to pass the `--werl` option when starting
-  IEx, such as `iex --werl` (or `iex.bat --werl` if using PowerShell).
-  `--werl` may be permanently enabled by setting the `IEX_WITH_WERL`
-  environment variable to `1`.
 
   ## Encoding and coloring
 
@@ -323,12 +323,14 @@ defmodule IEx do
   Connecting an Elixir shell to a remote node without Elixir is
   **not** supported.
 
+  When remsh halts, the Elixir shell process exits with reason `:normal`.
+
   ## The .iex.exs file
 
-  When starting, IEx looks for a local `.iex.exs` file (located in the current
-  working directory), then for a global `.iex.exs` file located inside the
-  directory pointed by the `IEX_HOME` environment variable (which defaults
-  to `~`) and loads the first one it finds (if any).
+  When starting, IEx looks for a configured path, then for a local `.iex.exs` file
+  (located in the current working directory), then for a global `.iex.exs` file
+  located inside the directory pointed by the `IEX_HOME` environment variable
+  (which defaults to `~`) and loads the first one it finds (if any).
 
   The code in the chosen `.iex.exs` file is evaluated line by line in the shell's
   context, as if each line were being typed in the shell. For instance, any modules
@@ -360,8 +362,9 @@ defmodule IEx do
       iex(1)> value
       13
 
-  It is possible to load another file by supplying the `--dot-iex` option
-  to IEx. See `iex --help`.
+  It is possible to load another file by configuring the `iex` application's `dot_iex`
+  value (`config :iex, dot_iex: "PATH"` or `IEx.configure(dot_iex: "PATH")`)
+  or supplying the `--dot-iex` option to IEx. See `iex --help`.
 
   In case of remote nodes, the location of the `.iex.exs` files are taken
   relative to the user that started the application, not to the user that
@@ -389,20 +392,92 @@ defmodule IEx do
 
   """
 
+  @typedoc """
+  Color settings used by the IEx shell.
+  """
+  @type colors_opts :: [
+          enabled: boolean(),
+          eval_result: IO.ANSI.ansidata(),
+          eval_info: IO.ANSI.ansidata(),
+          eval_error: IO.ANSI.ansidata(),
+          eval_interrupt: IO.ANSI.ansidata(),
+          stack_info: IO.ANSI.ansidata(),
+          blame_diff: IO.ANSI.ansidata(),
+          ls_directory: IO.ANSI.ansidata(),
+          ls_device: IO.ANSI.ansidata(),
+          doc_code: IO.ANSI.ansidata(),
+          doc_inline_code: IO.ANSI.ansidata(),
+          doc_headings: IO.ANSI.ansidata(),
+          doc_title: IO.ANSI.ansidata(),
+          doc_bold: IO.ANSI.ansidata(),
+          doc_underline: IO.ANSI.ansidata(),
+          syntax_colors: syntax_colors_opts() | false
+        ]
+
+  @typedoc """
+  Syntax coloring settings for inspected expressions.
+  """
+  @type syntax_colors_opts :: [
+          atom: IO.ANSI.ansidata(),
+          binary: IO.ANSI.ansidata(),
+          boolean: IO.ANSI.ansidata(),
+          charlist: IO.ANSI.ansidata(),
+          list: IO.ANSI.ansidata(),
+          map: IO.ANSI.ansidata(),
+          nil: IO.ANSI.ansidata(),
+          number: IO.ANSI.ansidata(),
+          string: IO.ANSI.ansidata(),
+          tuple: IO.ANSI.ansidata(),
+          variable: IO.ANSI.ansidata(),
+          call: IO.ANSI.ansidata(),
+          operator: IO.ANSI.ansidata(),
+          reset: IO.ANSI.ansidata()
+        ]
+
+  @typedoc """
+  Inspect settings used by the IEx shell.
+  """
+  @type inspect_opts :: [
+          base: :binary | :decimal | :octal | :hex,
+          binaries: :infer | :as_binaries | :as_strings,
+          charlists: :infer | :as_charlists | :as_lists,
+          custom_options: keyword(),
+          inspect_fun: (term(), Inspect.Opts.t() -> Inspect.Algebra.t()),
+          limit: pos_integer() | :infinity,
+          pretty: boolean(),
+          printable_limit: pos_integer() | :infinity,
+          safe: boolean(),
+          structs: boolean(),
+          syntax_colors: syntax_colors_opts(),
+          width: pos_integer() | :infinity
+        ]
+
+  @type configure_opts :: [
+          auto_reload: boolean(),
+          alive_prompt: String.t(),
+          colors: colors_opts(),
+          default_prompt: String.t(),
+          dot_iex: String.t() | nil,
+          history_size: integer(),
+          inspect: inspect_opts(),
+          parser: {module(), atom(), [any()]},
+          width: pos_integer()
+        ]
+
   @doc """
   Configures IEx.
 
   The supported options are:
 
-    * `:colors`
-    * `:inspect`
-    * `:width`
-    * `:history_size`
-    * `:default_prompt`
-    * `:continuation_prompt`
+    * `:auto_reload`
     * `:alive_prompt`
-    * `:alive_continuation_prompt`
+    * `:colors`
+    * `:default_prompt`
+    * `:dot_iex`
+    * `:history_size`
+    * `:inspect`
     * `:parser`
+    * `:width`
 
   They are discussed individually in the sections below.
 
@@ -482,14 +557,8 @@ defmodule IEx do
 
     * `:default_prompt` - used when `Node.alive?/0` returns `false`
 
-    * `:continuation_prompt` - used when `Node.alive?/0` returns `false`
-      and more input is expected
-
     * `:alive_prompt` - used when `Node.alive?/0` returns `true`
 
-    * `:alive_continuation_prompt` - used when `Node.alive?/0` returns
-      `true` and more input is expected
-
   The following values in the prompt string will be replaced appropriately:
 
     * `%counter` - the index of the history
@@ -503,13 +572,30 @@ defmodule IEx do
   The parser is a "mfargs", which is a tuple with three elements:
   the module name, the function name, and extra arguments to
   be appended. The parser receives at least three arguments, the
-  current input as a string, the parsing options as a keyword list,
-  and the buffer as a string. It must return `{:ok, expr, buffer}`
-  or `{:incomplete, buffer}`.
+  current input as a charlist, the parsing options as a keyword list,
+  and the state. The initial state is an empty charlist. It must
+  return `{:ok, expr, state}` or `{:incomplete, state}`.
 
-  If the parser raises, the buffer is reset to an empty string.
+  If the parser raises, the state is reset to an empty charlist.
+
+  > In earlier Elixir versions, the parser would receive the input
+  > and the initial buffer as strings. However, this behaviour
+  > changed when Erlang/OTP introduced multiline editing. If you
+  > support earlier Elixir versions, you can normalize the inputs
+  > by calling `to_charlist/1`.
+
+  ## `.iex`
+
+  Configure the file loaded into your IEx session when it starts.
+  See more information [in the `.iex.exs` documentation](`m:IEx#module-the-iex-exs-file`).
+
+  ## Auto reloading
+
+  When set to `true`, the `:auto_reload` option automatically purges
+  in-memory modules when they get invalidated by a concurrent compilation
+  happening in the Operating System.
   """
-  @spec configure(keyword()) :: :ok
+  @spec configure(configure_opts) :: :ok
   def configure(options) do
     IEx.Config.configure(options)
   end
@@ -713,7 +799,7 @@ defmodule IEx do
   defp __break__!(ast, module, fun, args, guards, stops, env) do
     module = Macro.expand(module, env)
 
-    unless is_atom(module) do
+    if not is_atom(module) do
       raise_unknown_break_ast!(ast)
     end
 
