@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../test_helper.exs", __DIR__)
 
 defmodule IEx.AutocompleteTest do
@@ -13,8 +17,8 @@ defmodule IEx.AutocompleteTest do
     ExUnit.CaptureIO.capture_io(fn ->
       evaluator = Process.get(:evaluator)
       Process.group_leader(evaluator, Process.group_leader())
-      send(evaluator, {:eval, self(), line <> "\n", 1, ""})
-      assert_receive {:evaled, _, _, _}
+      send(evaluator, {:eval, self(), Code.string_to_quoted!(line <> "\n"), 1})
+      assert_receive {:evaled, _, _}
     end)
   end
 
@@ -58,7 +62,16 @@ defmodule IEx.AutocompleteTest do
     assert expand(~c"t String") == {:yes, ~c"", [~c"String", ~c"StringIO"]}
 
     assert expand(~c"t String.") ==
-             {:yes, ~c"", [~c"codepoint/0", ~c"grapheme/0", ~c"pattern/0", ~c"t/0"]}
+             {:yes, ~c"",
+              [
+                ~c"codepoint/0",
+                ~c"grapheme/0",
+                ~c"pattern/0",
+                ~c"replace_opts/0",
+                ~c"split_opts/0",
+                ~c"splitter_opts/0",
+                ~c"t/0"
+              ]}
 
     assert expand(~c"t String.grap") == {:yes, ~c"heme", []}
     assert expand(~c"t  String.grap") == {:yes, ~c"heme", []}
@@ -81,7 +94,16 @@ defmodule IEx.AutocompleteTest do
     assert expand(~c"t(String") == {:yes, ~c"", [~c"String", ~c"StringIO"]}
 
     assert expand(~c"t(String.") ==
-             {:yes, ~c"", [~c"codepoint/0", ~c"grapheme/0", ~c"pattern/0", ~c"t/0"]}
+             {:yes, ~c"",
+              [
+                ~c"codepoint/0",
+                ~c"grapheme/0",
+                ~c"pattern/0",
+                ~c"replace_opts/0",
+                ~c"split_opts/0",
+                ~c"splitter_opts/0",
+                ~c"t/0"
+              ]}
 
     assert expand(~c"t(String.grap") == {:yes, ~c"heme", []}
   end
@@ -180,11 +202,36 @@ defmodule IEx.AutocompleteTest do
     assert expand(~c"IEx.Xyz") == {:no, ~c"", []}
   end
 
+  test "block keywords" do
+    assert expand(~c"if true do") == {:yes, ~c"", [~c"do"]}
+    assert expand(~c"if true a") == {:yes, ~c"", [~c"after", ~c"and/2"]}
+    assert expand(~c"if true d") == {:yes, ~c"o", []}
+    assert expand(~c"if true e") == {:yes, ~c"", [~c"end", ~c"else"]}
+  end
+
+  test "block keywords or operators" do
+    {:yes, ~c"", hints} = expand(~c"if true ")
+    assert ~c"do" in hints
+    assert ~c"else" in hints
+    assert ~c"+/2" in hints
+    assert ~c"and/2" in hints
+  end
+
   test "function completion" do
     assert expand(~c"System.ve") == {:yes, ~c"rsion", []}
     assert expand(~c":ets.fun2") == {:yes, ~c"ms", []}
   end
 
+  test "function completion with groups" do
+    {:yes, ~c"", [exports, guards]} = expand(~c"Kernel.i")
+    assert %{title: ~c"Exports", elems: [~c"if/2", ~c"inspect/1", ~c"inspect/2"]} = exports
+    assert %{title: ~c"Guards", elems: [_ | _]} = guards
+
+    {:yes, ~c"", [guards, exports]} = expand(~c"Kernel.in")
+    assert %{title: ~c"Guards", elems: [~c"in/2"]} = guards
+    assert %{title: ~c"Exports", elems: [~c"inspect/1", ~c"inspect/2"]} = exports
+  end
+
   test "function completion with arity" do
     assert expand(~c"String.printable?") == {:yes, ~c"", [~c"printable?/1", ~c"printable?/2"]}
     assert expand(~c"String.printable?/") == {:yes, ~c"", [~c"printable?/1", ~c"printable?/2"]}
@@ -416,6 +463,9 @@ defmodule IEx.AutocompleteTest do
     assert {:yes, ~c"ry: ", []} = expand(~c"%URI{path: \"foo\", que")
     assert {:no, [], []} = expand(~c"%URI{path: \"foo\", unkno")
     assert {:no, [], []} = expand(~c"%Unknown{path: \"foo\", unkno")
+
+    assert {:yes, [], _} = expand(~c"%__MODULE__{")
+    assert {:yes, [], _} = expand(~c"%__MODULE__.Some{")
   end
 
   test "completion for struct keys in update syntax" do
