@@ -1,8 +1,35 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Mix.Shell do
   @moduledoc """
   Defines `Mix.Shell` contract.
   """
 
+  @type cmd_opts :: [
+          {:print_app, boolean()}
+          | {:stderr_to_stdout, boolean()}
+          | {:quiet, boolean()}
+          | {:env, [{String.t(), String.t()}]}
+          | {:cd, String.t()}
+          | {atom(), term()}
+        ]
+
+  @type yes_opts :: [
+          {:default, :yes | :no}
+          | {atom(), term()}
+        ]
+
+  @type stream_cmd_opts :: [
+          {:cd, String.t()}
+          | {:stderr_to_stdout, boolean()}
+          | {:use_stdio, boolean()}
+          | {:env, [{String.t(), String.t()}]}
+          | {:quiet, boolean()}
+          | {atom(), term()}
+        ]
+
   @doc false
   defstruct [:callback]
 
@@ -44,7 +71,7 @@ defmodule Mix.Shell do
 
   All the built-in shells support these.
   """
-  @callback cmd(command :: String.t(), options :: keyword) :: integer
+  @callback cmd(command :: String.t(), options :: cmd_opts) :: integer
 
   @doc """
   Prompts the user for input.
@@ -65,7 +92,7 @@ defmodule Mix.Shell do
 
     * `:default` - `:yes` or `:no` (the default is `:yes`)
   """
-  @callback yes?(message :: binary, options :: keyword) :: boolean
+  @callback yes?(message :: binary, options :: yes_opts) :: boolean
 
   @doc """
   Prints the current application to the shell if
@@ -90,29 +117,31 @@ defmodule Mix.Shell do
   end
 
   @doc """
-  Executes the given `command` as a shell command and
-  invokes the `callback` for the streamed response.
+  Executes the given `command` and invokes the `callback` for the streamed response.
 
-  This is most commonly used by shell implementations
-  but can also be invoked directly.
+  `command` is either a string, to be executed as a `System.shell/2` command,
+  or a `{executable, args}` to be executed via `System.cmd/3`.
 
-  `callback` takes the output data of the command. Its
-  return value is ignored.
+  `callback` takes the output data of the command. Its return value is ignored.
+
+  This function is most commonly used by `Mix.Shell` implementations but can
+  also be invoked directly.
 
   ## Options
 
     * `:cd` *(since v1.11.0)* - the directory to run the command in
 
-    * `:stderr_to_stdout` - redirects stderr to stdout, defaults to true, unless use_stdio is set to false
+    * `:stderr_to_stdout` - redirects stderr to stdout, defaults to `true`, unless `:use_stdio` is set to `false`
 
-    * `:use_stdio` - controls whether the command should use stdin / stdout / stdrr, defaults to true
+    * `:use_stdio` - controls whether the command should use `stdin` / `stdout` / `stdrr`, defaults to `true`
 
     * `:env` - a list of environment variables, defaults to `[]`
 
     * `:quiet` - overrides the callback to no-op
 
   """
-  @spec cmd(String.t(), keyword, (binary -> term)) :: exit_status :: non_neg_integer
+  @spec cmd(String.t() | {String.t(), [String.t()]}, stream_cmd_opts, (binary -> term)) ::
+          exit_status :: non_neg_integer
   def cmd(command, options \\ [], callback) when is_function(callback, 1) do
     callback =
       if options[:quiet] do
@@ -129,7 +158,12 @@ defmodule Mix.Shell do
       |> Keyword.put(:into, %Mix.Shell{callback: callback})
       |> Keyword.put_new(:stderr_to_stdout, use_stdio)
 
-    {_, status} = System.shell(command, options)
+    {_, status} =
+      case command do
+        {command, args} -> System.cmd(command, args, options)
+        command when is_binary(command) -> System.shell(command, options)
+      end
+
     status
   end
 
