@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Mix.Shell.IO do
   @moduledoc """
   This is Mix's default shell.
@@ -7,6 +11,11 @@ defmodule Mix.Shell.IO do
 
   @behaviour Mix.Shell
 
+  @typedoc """
+  Options for `yes?/2`.
+  """
+  @type yes_opts :: [default: :yes | :no]
+
   @doc """
   Prints the current application to the shell if it
   was not printed yet.
@@ -68,6 +77,7 @@ defmodule Mix.Shell.IO do
       end
 
   """
+  @spec yes?(String.t(), yes_opts) :: boolean()
   def yes?(message, options \\ []) do
     default = Keyword.get(options, :default, :yes)
 
@@ -103,9 +113,7 @@ defmodule Mix.Shell.IO do
 
     Mix.Shell.cmd(command, opts, fn data ->
       if print_app?, do: print_app()
-      # Due to encoding of shell command on Windows,
-      # let's write the data as is
-      IO.binwrite(data)
+      IO.write(data)
     end)
   end
 end
