@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Mix.Task do
   @moduledoc """
   Provides conveniences for creating, loading, and manipulating Mix tasks.
@@ -397,7 +401,7 @@ defmodule Mix.Task do
   @doc since: "1.14.0"
   @spec run_in_apps(task_name, [atom], [any]) :: any
   def run_in_apps(task, apps, args \\ []) do
-    unless Mix.Project.umbrella?() do
+    if !Mix.Project.umbrella?() do
       Mix.raise(
         "Could not run #{inspect(task)} with the --app option because this is not an umbrella project"
       )
@@ -520,16 +524,24 @@ defmodule Mix.Task do
         shell.info(["<- Ran mix ", task, " in ", Integer.to_string(div(time, 1000)), "ms"])
         res
 
-      task in Mix.State.get(:profile, []) ->
+      opts = profile_opts_for(task) ->
         shell = Mix.shell()
         shell.info(["-> Profiling mix ", task_to_string(task, args), project_to_string(proj)])
-        Mix.Tasks.Profile.Eprof.profile(fun, warmup: false, set_on_spawn: false)
+        Mix.Tasks.Profile.Tprof.profile(fun, opts)
 
       true ->
         fun.()
     end
   end
 
+  defp profile_opts_for(task) do
+    {opts, tasks} = Mix.State.get(:profile, {[], []})
+
+    if task in tasks do
+      opts
+    end
+  end
+
   defp project_to_string(nil), do: ""
   defp project_to_string(proj), do: " (inside #{inspect(proj)})"
 
