@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Mix.Tasks.App.Start do
   use Mix.Task
 
@@ -6,19 +10,22 @@ defmodule Mix.Tasks.App.Start do
   @moduledoc """
   Starts all registered apps.
 
-  First this task guarantees that all dependencies are in place
-  and that the current project has been compiled. Then the current
+  First, this task guarantees that all dependencies are in place
+  and that the current project has been compiled. Then, the current
   application is started as a temporary application, unless
   `:start_permanent` is set to `true` in your project configuration
-  or the `--permanent` option is given, then it's started as permanent,
-  which guarantees the node will shut down if the application
-  crashes permanently.
+  or the `--permanent` option is given. Setting it to permanent
+  guarantees the node will shut down if the application terminates
+  (typically because its root supervisor has terminated).
 
   ## Configuration
 
     * `:start_permanent` - the application and all of its children
       applications are started in permanent mode. Defaults to `false`.
 
+    * `:start_concurrently` - applications are started concurrently
+      whenever possible. Defaults to `false`.
+
   ## Command line options
 
     * `--force` - forces compilation regardless of compilation times
@@ -53,20 +60,35 @@ defmodule Mix.Tasks.App.Start do
       # application to decide if it should be restarted or not.
       #
       # Mix should not depend directly on Logger so check that it's loaded.
-      if Process.whereis(Logger) do
-        Logger.App.stop()
-      end
-
+      Logger.App.stop()
       config = Mix.Project.config()
-      start(apps(config), type(config, opts))
+      type = type(config, opts)
+      mode = if config[:start_concurrently], do: :concurrent, else: :serial
+      start(apps(config), type, mode)
     end
 
     :ok
   end
 
   @doc false
-  def start(apps, type) do
-    Enum.each(apps, &ensure_all_started(&1, type))
+  def start(apps, type, mode) do
+    case Application.ensure_all_started(apps, type: type, mode: mode) do
+      {:ok, _} ->
+        :ok
+
+      {:error, {app, reason}} when type == :permanent ->
+        # We need to stop immediately because application_controller is
+        # shutting down all applications. Since any work we do here is prone
+        # to race conditions as whatever process we call may no longer exist,
+        # we print a quick message, and then we block by calling `System.stop/1`.
+        Mix.shell().error(["** (Mix) ", could_not_start(app, reason)])
+        System.stop(1)
+        Process.sleep(:infinity)
+
+      {:error, {app, reason}} ->
+        Mix.raise(could_not_start(app, reason))
+    end
+
     :ok
   end
 
@@ -78,31 +100,6 @@ defmodule Mix.Tasks.App.Start do
     end
   end
 
-  defp ensure_all_started(app, type) do
-    case Application.start(app, type) do
-      :ok ->
-        :ok
-
-      {:error, {:already_started, ^app}} ->
-        :ok
-
-      {:error, {:not_started, dep}} ->
-        :ok = ensure_all_started(dep, type)
-        ensure_all_started(app, type)
-
-      {:error, reason} when type == :permanent ->
-        # We need to stop immediately because application_controller is
-        # shutting down all applications. Since any work we do here is prone
-        # to race conditions as whatever process we call may no longer exist,
-        # we print a quick message and then block by calling `System.stop/1`.
-        Mix.shell().error(["** (Mix) ", could_not_start(app, reason)])
-        System.stop(1)
-
-      {:error, reason} ->
-        Mix.raise(could_not_start(app, reason))
-    end
-  end
-
   defp could_not_start(app, reason) do
     "Could not start application #{app}: " <> Application.format_error(reason)
   end
