@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Mix.Tasks.Deps.Tree do
   use Mix.Task
 
@@ -13,13 +17,18 @@ defmodule Mix.Tasks.Deps.Tree do
 
   ## Command line options
 
-    * `--only` - the environment to show dependencies for
+    * `--only` - the environment to show dependencies for.
 
-    * `--target` - the target to show dependencies for
+    * `--target` - the target to show dependencies for.
 
-    * `--exclude` - exclude dependencies which you do not want to see printed.
+    * `--exclude` - exclude dependencies which you do not want to see printed. You can
+      pass this flag multiple times to exclude multiple dependencies.
 
-    * `--format` - Can be set to one of either:
+    * `--umbrella-only` *(since v1.17.0)* - only include the umbrella applications.
+      This is useful when you have an umbrella project and want to see the
+      relationship between the apps in the project.
+
+    * `--format` - can be set to one of either:
 
       * `pretty` - uses Unicode code points for formatting the tree.
         This is the default except on Windows.
@@ -31,8 +40,29 @@ defmodule Mix.Tasks.Deps.Tree do
         in `deps_tree.dot` in the current directory.
         Warning: this will override any previously generated file.
 
+  ## Examples
+
+  Print the dependency tree for the `:test` environment:
+
+      $ mix deps.tree --only test
+
+  Exclude the `:ecto` and `:phoenix` dependencies:
+
+      $ mix deps.tree --exclude ecto --exclude phoenix
+
+  Only print the tree for the `:bandit` and `:phoenix` dependencies:
+
+      $ mix deps.tree --include bandit --include phoenix
+
   """
-  @switches [only: :string, target: :string, exclude: :keep, format: :string]
+
+  @switches [
+    only: :string,
+    target: :string,
+    exclude: :keep,
+    umbrella_only: :boolean,
+    format: :string
+  ]
 
   @impl true
   def run(args) do
@@ -77,7 +107,13 @@ defmodule Mix.Tasks.Deps.Tree do
   end
 
   defp callback(formatter, deps, opts) do
+    umbrella_only? = Keyword.get(opts, :umbrella_only, false)
     excluded = Keyword.get_values(opts, :exclude) |> Enum.map(&String.to_atom/1)
+
+    if umbrella_only? && !Mix.Project.parent_umbrella_project_file() do
+      Mix.raise("The --umbrella-only option can only be used in umbrella projects")
+    end
+
     top_level = Enum.filter(deps, & &1.top_level)
 
     fn
@@ -91,19 +127,28 @@ defmodule Mix.Tasks.Deps.Tree do
             find_dep(deps, app).deps
           end
 
-        {formatter.(dep), exclude_and_sort(deps, excluded)}
+        {formatter.(dep), select_and_sort(deps, excluded, umbrella_only?)}
 
       app ->
-        {{Atom.to_string(app), nil}, exclude_and_sort(top_level, excluded)}
+        {{Atom.to_string(app), nil}, select_and_sort(top_level, excluded, umbrella_only?)}
     end
   end
 
-  defp exclude_and_sort(deps, excluded) do
+  defp select_and_sort(deps, excluded, umbrella_only?) do
     deps
+    |> filter_umbrella_only(umbrella_only?)
     |> Enum.reject(&(&1.app in excluded))
     |> Enum.sort_by(& &1.app)
   end
 
+  defp filter_umbrella_only(deps, umbrella_only?) do
+    if umbrella_only? do
+      Enum.filter(deps, fn %Mix.Dep{opts: opts} -> opts[:in_umbrella] end)
+    else
+      deps
+    end
+  end
+
   defp format_dot(%{app: app, requirement: requirement, opts: opts}) do
     override =
       if opts[:override] do
