@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../test_helper.exs", __DIR__)
 
 defmodule Mix.DepTest do
@@ -295,12 +299,11 @@ defmodule Mix.DepTest do
 
         Mix.Tasks.Deps.run([])
         assert_received {:mix_shell, :info, ["* deps_repo1" <> _]}
-        assert_received {:mix_shell, :info, [_]}
         assert_received {:mix_shell, :info, ["* deps_repo2" <> _]}
-        assert_received {:mix_shell, :info, [_]}
         assert_received {:mix_shell, :info, ["* git_repo" <> _]}
-        assert_received {:mix_shell, :info, [msg]}
-        assert msg =~ "different specs were given for the git_repo"
+
+        assert_received {:mix_shell, :info,
+                         ["  different specs were given for the git_repo app" <> _]}
       end)
     end)
   end
@@ -343,12 +346,11 @@ defmodule Mix.DepTest do
 
         Mix.Tasks.Deps.run([])
         assert_received {:mix_shell, :info, ["* deps_repo1" <> _]}
-        assert_received {:mix_shell, :info, [_]}
         assert_received {:mix_shell, :info, ["* deps_repo2" <> _]}
-        assert_received {:mix_shell, :info, [_]}
         assert_received {:mix_shell, :info, ["* git_repo" <> _]}
-        assert_received {:mix_shell, :info, [msg]}
-        assert msg =~ "different specs were given for the git_repo"
+
+        assert_received {:mix_shell, :info,
+                         ["  different specs were given for the git_repo app" <> _]}
       end)
     end)
   end
@@ -944,7 +946,7 @@ defmodule Mix.DepTest do
 
           Mix.Tasks.Deps.Get.run([])
           Mix.Tasks.Deps.Compile.run([])
-          refute_receive {:mix_shell, :error, ["Could not compile :git_repo" <> _]}
+          refute_receive {:mix_shell, :error, ["Could not compile :git_repo" <> _]}, 100
         end)
       end)
     end
