@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../../test_helper.exs", __DIR__)
 
 defmodule Mix.SCM.GitTest do
@@ -32,7 +36,7 @@ defmodule Mix.SCM.GitTest do
              "https://github.com/elixir-lang/some_dep.git - v1"
 
     assert Mix.SCM.Git.format(Keyword.put(opts, :branch, "b")) ==
-             "https://github.com/elixir-lang/some_dep.git - origin/b"
+             "https://github.com/elixir-lang/some_dep.git - b"
 
     assert Mix.SCM.Git.format(Keyword.put(opts, :ref, "abcdef")) ==
              "https://github.com/elixir-lang/some_dep.git - abcdef"
